<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/*
	Class: Cron

	Runs the various cron tasks enabled in the system.

	Cronjobs are called 'Tasks' and are available both to the Core, the
	application itself, and to each module. Tasks are basically libraries
	that contain methods, one per task. Each method name must be prefixed
	with 'task_'.

	Non-module tasks are stored	in application/libraries. Each module may have
	a tasks.php file within the libraries folder that stores the Tasks class.

	The file/class should be named by the module name, an underscore, then tasks.
	Both the filename and the classname should be lowercase.

	For a blog module, it would be:

		File	- libraries/blog_tasks.php
		Class	- blog_tasks
*/
class Cron extends Base_Controller {

	public function __construct()
	{
		parent::__construct();

		// Make sure we have plenty of time to run large tasks.
		set_time_limit(0);
	}

	//--------------------------------------------------------------------


	/*
		Method: _remap()

		Redirects all traffic to the index method.
	*/
	public function _remap($method=null)
	{
		$this->index();
	}

	//--------------------------------------------------------------------

	/*
		Method: index()

		Handles running all of the tasks in the system.
	*/
	/**
	 * CRONJOB WILL RUN AFTER A MINUTE OF THE SCHEDULED TIME
	 * cron-expression next run time is updated on current timestamp
	 * e.g. if current time is 2016-Jan-09 08:39 and Next Run is On 2016-Jan-09 08:40
	 * Then on 08:40 Next Run will be updated to 2016-Jan-09 08:45
	 * Scheduled time and current time are not good to match
	 * because this will result on multiple times run of the same job
	 * No Benefit to Add or Subtract minute also
	 */
	public function index()
	{
		$run_time = date('Y-M-d H:i');
		if (!isset($this->db)) {
			$this->load->database();
		}
		$this->load->model('cronjobs/cronjob_model');
		$tasks = $this->cronjob_model->select('id, module, time_minutes, time_hours, time_week_day, time_month_day, time_month, enabled, method_name, running, last_stop')->find_all();
		if ($tasks) {
			foreach ($tasks as $task) {
				// Is it already running?
				if ($task->running == 1) {
					continue;
				}
				if (!$task->enabled) {
						continue;
				}
					
				// Calculate the next time this is scheduled to run
				$scheduled_time = $this->cronjob_model->calc_next_run($task);
				$scheduled_time = date('Y-M-d H:i', $scheduled_time);
				$previous_run = $this->cronjob_model->calc_prev_run($task);
	
				/**
				 * Time to actually run this job!
				 * Compare Previous time with the Task Last Stop Value
				 * Task will only run if the Previous Time is greater than the Last Stop value. 
				 * Task will also run once.
				 */
				if (($previous_run > $task->last_stop)) {	
					// Record we're running.
					$this->cronjob_model->update($task->id, array('running' => 1, 'last_start' => time()));	
					$task->module = strtolower($task->module);
						
					// Load the task library!
					if ($task->module == 'core') {
						$this->load->library('core_tasks');
						$task_lib = $this->core_tasks;
					} else {
						$this->load->library($task->module .'/'. $task->module .'_tasks');
						$name = $task->module .'_tasks';
						$task_lib = $this->$name;	
						unset($name);
					}
	
					// Run the task
					if (method_exists($task_lib, $task->method_name)) {
						$task_lib->{$task->method_name}();
					}
	
					// Record we're not running.
					$this->cronjob_model->update($task->id, array('running' => 0, 'last_stop' => time() ));
				}
			}
		}
		// Record our last run time
		$this->settings_lib->set('cron_last_run', date('Y-m-d H:i:s'), 'cronjobs');
	}

	//--------------------------------------------------------------------

	//--------------------------------------------------------------------
	// !Private Methods
	//--------------------------------------------------------------------

}

// End Cron controller

//--------------------------------------------------------------------

/*
	Class: Base_Task

	Provides simple reporting methods that all task classes should
	extend from.
*/
class Base_Task {

	/*
		Var: $ci
		Stores a pointer to the CI superobject.
	*/
	protected $ci;

	/*
		Var: $format
		Either html or cli
	*/
	protected $format;

	//--------------------------------------------------------------------

	public function __construct()
	{
		$this->ci =& get_instance();

		$this->format = $this->ci->input->is_cli_request() ? 'cli' : 'html';
	}

	//--------------------------------------------------------------------

	/*
		Method: output_header()

		Outputs a header to the screen.

		Parameters:
			$task_name	- A string to be displayed in the header that
						  is the name of the task being ran.
	*/
	protected function output_header($task_name='Unnamed Task')
	{
		if ($this->format == 'cli')
		{
			$tpl =<<<EOL
//--------------------------------------------------------------------
// Starting {task_name} - {time}
//--------------------------------------------------------------------

EOL;
		}
		else
		{
			$tpl =<<<EOL
//--------------------------------------------------------------------<br/>
// Starting {task_name} - {time}<br/>
//--------------------------------------------------------------------<br/>
<br/>
EOL;
		}

		$output = str_replace('{task_name}', $task_name, $tpl);
		$output = str_replace('{time}', date($this->ci->config->item('log_date_format')), $output);

		echo $output;
	}

	//--------------------------------------------------------------------

	/*
		Method: output_footer()

		Outputs a footer to the screen.

		Parameters:
			$task_name	- A string to be displayed in the footer that
						  is the name of the task being ran.
	*/
	protected function output_footer($task_name='Unnamed Task')
	{
		if ($this->format == 'cli')
		{
			$tpl =<<<EOL

// {task_name} Done - {time}
//--------------------------------------------------------------------
EOL;
		}
		else
		{
			$tpl =<<<EOL
<br/>
// {task_name} Done - {time}<br/>
//--------------------------------------------------------------------<br/><br/>
EOL;
		}

		$output = str_replace('{task_name}', $task_name, $tpl);
		$output = str_replace('{time}', date($this->ci->config->item('log_date_format')), $output);

		echo $output;

		// Trigger a save to the database to show that we're complete
	}

	//--------------------------------------------------------------------

}

// End Base_Task class