<?php defined('BASEPATH') || exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers to jumpstart their development of
 * CodeIgniter applications
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2015, Bonfire Dev Team
 * @license   http://opensource.org/licenses/MIT The MIT License
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

/**
 * Application Hooks
 *
 * A set of methods used for the CodeIgniter hooks.
 * @link https://ellislab.com/codeigniter/user-guide/general/hooks.html
 *
 * @package Bonfire\Hooks\App_hooks
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs/developer
 */
class App_hooks
{
    /** @var array List of pages which bypass the Site Offline page. */
    protected $allowOffline = array(
        //'/users/login',
        //'/users/logout',
    	'/admin/login', 
        '/users/restrictedZone',
        '/ajax/offline' // Site Offline Form
    );

    protected $isInstalled = false;

    /**
     * @var object The CodeIgniter core object.
     */
    private $ci;

    /**
     * @var array List of pages for which the URL-save/prep hooks are not run.
     */
    private $ignore_pages = array(
        '/users/login',
        '/users/logout',
        '/users/register',
        '/users/forgot_password',
        '/users/activate',
        '/users/resend_activation',
        '/images',
    );

    //--------------------------------------------------------------------------

    /**
     * Constructor
     *
     * @return void
     */
    public function __construct()
    {
        $this->ci =& get_instance();

        if (is_object($this->ci)) {
            $this->isInstalled = $this->ci->config->item('bonfire.installed');
            if (! $this->isInstalled) {
                // Is Bonfire installed?
                $this->ci->load->library('installer_lib');
                $this->isInstalled = $this->ci->installer_lib->is_installed();
            }
        }
    }

    /**
     * Check the online/offline status of the site.
     *
     * Called by the "post_controller_constructor" hook.
     *
     * @return void
     */
    public function checkSiteStatus()
    {
        if (! isset($this->ci->load)) {
            return;
        }

        // If the settings lib is not available, load it.
        if (! isset($this->ci->settings_lib)) {
            $this->ci->load->library('settings/settings_lib');
        }

        if ($this->ci->settings_lib->item('site.status') != 0) {
            return;
        }

        if (! class_exists('Auth', false)) {
            $this->ci->load->library('users/auth');
        }

        if (! $this->ci->auth->has_permission('Site.Signin.Offline')
            && ! $this->ruriInArray($this->allowOffline)
        ) {
            $offlineReason = $this->ci->settings_lib->item('site.offline_reason');
            include(APPPATH . 'errors/offline.php');
            die();
        }
    }

    /**
     * Stores the name of the current uri in the session as 'previous_page'.
     * This allows redirects to take us back to the previous page without
     * relying on inconsistent browser support or spoofing.
     *
     * Called by the "post_controller" hook.
     *
     * @return void
     */
    public function prepRedirect()
    {
        if (! class_exists('CI_Session', false)) {
            $this->ci->load->library('session');
        }

        if (! $this->ruriInArray($this->ignore_pages)) {
            $this->ci->session->set_userdata('previous_page', current_url());
        }
    }

    /**
     * Store the requested page in the session data so we can use it
     * after the user logs in.
     *
     * Called by the "pre_controller" hook.
     *
     * @return void
     */
    public function saveRequested()
    {
        if (! $this->isInstalled) {
            return;
        }

        // If the CI_Session class is not loaded, this might be a controller that
        // doesn't extend any of Bonfire's controllers. In that case, try to do
        // this the old fashioned way and add it straight to the session.

        if (! class_exists('CI_Session', false)) {
            if (is_object(get_instance())) {
                // If an instance is available, just load the session lib.
                $this->ci->load->library('session');
            } elseif (get_instance() === null) {
                // If an instance is not available...
                // Try to grab the REQUEST_URI since this will work in most cases.
                $uri = empty($_SERVER['REQUEST_URI']) ? null : $_SERVER['REQUEST_URI'];
                if (empty($uri)) {
                    // Try to get the current URL through PATH INFO.
                    $path = isset($_SERVER['PATH_INFO']) ? $_SERVER['PATH_INFO'] : @getenv('PATH_INFO');
                    if (trim($path, '/') != '' && $path != '/' . SELF) {
                        $uri = $path;
                    }
                }

                if (empty($uri)) {
                    // Finally, try the query string.
                    $path =  isset($_SERVER['QUERY_STRING']) ? $_SERVER['QUERY_STRING'] : @getenv('QUERY_STRING');
                    if (trim($path, '/') != '') {
                        $uri = $path;
                    }
                }

                // Set the variable in the session and return.
                $_SESSION['requested_page'] = $uri;
                return;
            }
        }

        // Either the session library was available all along or it has been loaded,
        // so determine whether the current URL is in the ignore_pages array and,
        // if it is not, set it as the requested page in the session.

        if (! $this->ruriInArray($this->ignore_pages)) {
            $this->ci->session->set_userdata('requested_page', current_url());
        }
    }

    protected function ruriInArray(array $ruriArray)
    {
        // Output of uri->ruri_string() is considerably different in CI 3 when using
        // the BF_Router, so the following normalizes the output for the comparison
        // with $this->ignore_pages.
        $uri_string = $this->ci->uri->ruri_string() ? $this->ci->uri->ruri_string() : '';
        $uri_directory = $this->ci->router->directory ? $this->ci->router->directory : '';
        $ruriString = '/' . ltrim(
            str_replace($uri_directory, '', $uri_string),
            '/'
        );
        return in_array($ruriString, $ruriArray);
    }
    
    /**
     * Affiliate - Check for referal links
     * the ref is the referral_id in the commissions table
     * @return boolean
     */
    public function affiliate() {
        if($this->ci->input->get('ref', true)) {
            $this->ci->load->model('affiliates/affiliates_model');
            $this->ci->affiliates_model->direct_affiliate();
        }
    }
    
    /**
     * Affiliate - Check for referred by
     * @return boolean
     */
    public function referredBy() {
        if($this->ci->input->get('ref_by', true)) {
            $this->ci->load->model('affiliates/affiliates_model');
            $this->ci->affiliates_model->set_affiliate(array(), 'ref_by');
        }
    }
    
    /**
     * Affiliate - Check for affmid (affiliate material id) parameter in the URL
     * Sharing an affiliate URL on Facebook and LinkedIn should not result in affiliate and material id lost
     * The affmid will preserve the affiliate id on link share
     * @return boolean
     */
    public function checkAffmid() {
        if($this->ci->input->get('affmid', true)) {
            $this->ci->load->model('affiliates/affiliates_model');
            $this->ci->affiliates_model->cloaking($this->ci->input->get('affmid'));
        }
    }
    
    /**
     * Track and Block Visitors consuming resources unnecessarily
     * @return boolean
     */
    public function trackBlockVisitors() {
        /*
        // use in the public_html/index.php file to block users based on the HTTP_USER_AGENT
        $user_agent = $_SERVER['HTTP_USER_AGENT'];
        $str_find = array('BLEXBot', 'robot', 'PetalBot', 'dataforseo-bot', 'applebot', 'Bytespider'); // , 'Googlebot', 'bot.html'
        foreach($str_find as $str) {
            if(strpos($user_agent, $str) !== false) {
                die('Your IP Address has been blocked due to suspicious activities.');
                return false;
            }
        }
        $user_agent = $_SERVER['HTTP_USER_AGENT'];
        $str_find = array('.NET CLR 2.0.50727', 'BLEXBot', 'PetalBot', 'dataforseo-bot', 'applebot', 'Bytespider', 'AhrefsBot', 'SemrushBot', 'DotBot');
        foreach($str_find as $str) {
          if(strpos($user_agent, $str) !== false) {
            die('Your IP address has been blocked due to unusual activities.');
            return false;
          }
        }
        */
        if (! isset($this->ci->load)) {
            return;
        }
        $ip_address = $this->ci->input->ip_address();
        $ip_counts = $this->ci->db->where('ip_range', $ip_address)->count_all_results('visitor_restricted_ips');
        if($ip_counts > 0) {
            die('Your IP Address has been blocked due to suspicious activities.');
            return false;
        }
        $sql = 'SELECT COUNT(*) AS total_visits, created_ip FROM ' . $this->ci->db->dbprefix . 'visitor_tracking WHERE created_on >= NOW() - INTERVAL 1 MINUTE GROUP BY created_ip HAVING COUNT(total_visits) > 30 ORDER BY total_visits DESC';
        $records = $this->ci->db->query($sql)->result();
        if($records) {
            foreach($records as $record) {
                $sql = "INSERT IGNORE INTO " . $this->ci->db->dbprefix . "visitor_restricted_ips(ip_range) VALUES ('" . $record->created_ip . "')";
                $this->ci->db->query($sql);
            }
        }
        
        $this->ci->load->library('Mobile_Detect');
        $detect = new Mobile_Detect;
        $user_agent = $detect->getUserAgent();
        $additional_data = array(
            'web_browser' => $user_agent,
            'agent' => $_SERVER['HTTP_USER_AGENT']
        );
        $this->ci->db->insert('visitor_tracking', array('page_url' => current_url() . (!empty($_SERVER['QUERY_STRING']) ? '?' . $_SERVER['QUERY_STRING'] : ''), 'created_ip' => $this->ci->input->ip_address(), 'additional_info' => json_encode($additional_data)));

        // delete last 24 hours data
        $yesterday_date = date('Y-m-d', strtotime("yesterday"));
        $sql = "DELETE FROM " . $this->ci->db->dbprefix . "visitor_tracking WHERE DATE(created_on) < '" . $yesterday_date . "'";
        $this->ci->db->query($sql);
    }
}
