<?php

defined('BASEPATH') OR exit('No direct script access allowed');

/**
 * Format class
 * Help convert between various formats such as XML, JSON, CSV, etc.
 *
 * @author    Phil Sturgeon, Chris Kacerguis, @softwarespot
 * @license   http://www.dbad-license.org/
 */
// reference the Dompdf namespace
use Dompdf\Dompdf;
use Dompdf\Options;
use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
class Format {

    /**
     * Array output format
     */
    const ARRAY_FORMAT = 'array';

    /**
     * Comma Separated Value (CSV) output format
     */
    const CSV_FORMAT = 'csv';

    /**
     * Json output format
     */
    const JSON_FORMAT = 'json';

    /**
     * HTML output format
     */
    const HTML_FORMAT = 'html';

    /**
     * PHP output format
     */
    const PHP_FORMAT = 'php';

    /**
     * Serialized output format
     */
    const SERIALIZED_FORMAT = 'serialized';

    /**
     * XML output format
     */
    const XML_FORMAT = 'xml';

    /**
     * Default format of this class
     */
    const DEFAULT_FORMAT = self::JSON_FORMAT; // Couldn't be DEFAULT, as this is a keyword

    /**
     * CodeIgniter instance
     *
     * @var object
     */
    private $_CI;

    /**
     * Data to parse
     *
     * @var mixed
     */
    protected $_data = [];

    /**
     * Type to convert from
     *
     * @var string
     */
    protected $_from_type = NULL;

    /**
     * DO NOT CALL THIS DIRECTLY, USE factory()
     *
     * @param NULL $data
     * @param NULL $from_type
     * @throws Exception
     */

    public function __construct($data = NULL, $from_type = NULL)
    {
        // Get the CodeIgniter reference
        $this->_CI = &get_instance();

        // Load the inflector helper
        $this->_CI->load->helper('inflector');

        // If the provided data is already formatted we should probably convert it to an array
        if ($from_type !== NULL)
        {
            if (method_exists($this, '_from_'.$from_type))
            {
                $data = call_user_func([$this, '_from_'.$from_type], $data);
            }
            else
            {
                throw new Exception('Format class does not support conversion from "'.$from_type.'".');
            }
        }

        // Set the member variable to the data passed
        $this->_data = $data;
    }

    /**
     * Create an instance of the format class
     * e.g: echo $this->format->factory(['foo' => 'bar'])->to_csv();
     *
     * @param mixed $data Data to convert/parse
     * @param string $from_type Type to convert from e.g. json, csv, html
     *
     * @return object Instance of the format class
     */
    public function factory($data, $from_type = NULL)
    {
        // $class = __CLASS__;
        // return new $class();

        return new static($data, $from_type);
    }

    // FORMATTING OUTPUT ---------------------------------------------------------

    /**
     * Format data as an array
     *
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @return array Data parsed as an array; otherwise, an empty array
     */
    public function to_array($data = NULL)
    {
        // If no data is passed as a parameter, then use the data passed
        // via the constructor
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        // Cast as an array if not already
        if (is_array($data) === FALSE)
        {
            $data = (array) $data;
        }

        $array = [];
        foreach ((array) $data as $key => $value)
        {
            if (is_object($value) === TRUE || is_array($value) === TRUE)
            {
                $array[$key] = $this->to_array($value);
            }
            else
            {
                $array[$key] = $value;
            }
        }

        return $array;
    }

    /**
     * Format data as XML
     *
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @param NULL $structure
     * @param string $basenode
     * @return mixed
     */
    public function to_xml($data = NULL, $structure = NULL, $basenode = 'xml')
    {
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        if ($structure === NULL)
        {
            $structure = simplexml_load_string("<?xml version='1.0' encoding='utf-8'?><$basenode />");
        }

        // Force it to be something useful
        if (is_array($data) === FALSE && is_object($data) === FALSE)
        {
            $data = (array) $data;
        }

        foreach ($data as $key => $value)
        {

            //change false/true to 0/1
            if (is_bool($value))
            {
                $value = (int) $value;
            }

            // no numeric keys in our xml please!
            if (is_numeric($key))
            {
                // make string key...
                $key = (singular($basenode) != $basenode) ? singular($basenode) : 'item';
            }

            // replace anything not alpha numeric
            $key = preg_replace('/[^a-z_\-0-9]/i', '', $key);

            if ($key === '_attributes' && (is_array($value) || is_object($value)))
            {
                $attributes = $value;
                if (is_object($attributes))
                {
                    $attributes = get_object_vars($attributes);
                }

                foreach ($attributes as $attribute_name => $attribute_value)
                {
                    $structure->addAttribute($attribute_name, $attribute_value);
                }
            }
            // if there is another array found recursively call this function
            elseif (is_array($value) || is_object($value))
            {
                $node = $structure->addChild($key);

                // recursive call.
                $this->to_xml($value, $node, $key);
            }
            else
            {
                // add single node.
                $value = htmlspecialchars(html_entity_decode($value, ENT_QUOTES, 'UTF-8'), ENT_QUOTES, 'UTF-8');

                $structure->addChild($key, $value);
            }
        }

        return $structure->asXML();
    }

    /**
     * Format data as HTML
     *
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @return mixed
     */
    public function to_html($data = NULL)
    {
        // If no data is passed as a parameter, then use the data passed
        // via the constructor
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        // Cast as an array if not already
        if (is_array($data) === FALSE)
        {
            $data = (array) $data;
        }

        // Check if it's a multi-dimensional array
        if (isset($data[0]) && count($data) !== count($data, COUNT_RECURSIVE))
        {
            // Multi-dimensional array
            $headings = array_keys($data[0]);
        }
        else
        {
            // Single array
            $headings = array_keys($data);
            $data = [$data];
        }

        // Load the table library
        $this->_CI->load->library('table');

        $this->_CI->table->set_heading($headings);

        foreach ($data as $row)
        {
            // Suppressing the "array to string conversion" notice
            // Keep the "evil" @ here
            $row = @array_map('strval', $row);

            $this->_CI->table->add_row($row);
        }

        return $this->_CI->table->generate();
    }

    /**
     * @link http://www.metashock.de/2014/02/create-csv-file-in-memory-php/
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @param string $delimiter The optional delimiter parameter sets the field
     * delimiter (one character only). NULL will use the default value (,)
     * @param string $enclosure The optional enclosure parameter sets the field
     * enclosure (one character only). NULL will use the default value (")
     * @return string A csv string
     */
    public function to_csv($data = NULL, $delimiter = ',', $enclosure = '"')
    {
        // Use a threshold of 1 MB (1024 * 1024)
        $handle = fopen('php://temp/maxmemory:1048576', 'w');
        if ($handle === FALSE)
        {
            return NULL;
        }

        // If no data is passed as a parameter, then use the data passed
        // via the constructor
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        // If NULL, then set as the default delimiter
        if ($delimiter === NULL)
        {
            $delimiter = ',';
        }

        // If NULL, then set as the default enclosure
        if ($enclosure === NULL)
        {
            $enclosure = '"';
        }

        // Cast as an array if not already
        if (is_array($data) === FALSE)
        {
            $data = (array) $data;
        }

        // Check if it's a multi-dimensional array
        if (isset($data[0]) && count($data) !== count($data, COUNT_RECURSIVE))
        {
            // Multi-dimensional array
            $headings = array_keys($data[0]);
        }
        else
        {
            // Single array
            $headings = array_keys($data);
            $data = [$data];
        }

        // Apply the headings
        fputcsv($handle, $headings, $delimiter, $enclosure);

        foreach ($data as $record)
        {
            // If the record is not an array, then break. This is because the 2nd param of
            // fputcsv() should be an array
            if (is_array($record) === FALSE)
            {
                break;
            }

            // Suppressing the "array to string conversion" notice.
            // Keep the "evil" @ here.
            $record = @ array_map('strval', $record);

            // Returns the length of the string written or FALSE
            fputcsv($handle, $record, $delimiter, $enclosure);
        }

        // Reset the file pointer
        rewind($handle);

        // Retrieve the csv contents
        $csv = stream_get_contents($handle);

        // Close the handle
        fclose($handle);

        return $csv;
    }

    /**
     * Encode data as json
     *
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @return string Json representation of a value
     */
    public function to_json($data = NULL)
    {
        // If no data is passed as a parameter, then use the data passed
        // via the constructor
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        // Get the callback parameter (if set)
        $callback = $this->_CI->input->get('callback');

        if (empty($callback) === TRUE)
        {
            return json_encode($data);
        }

        // We only honour a jsonp callback which are valid javascript identifiers
        elseif (preg_match('/^[a-z_\$][a-z0-9\$_]*(\.[a-z_\$][a-z0-9\$_]*)*$/i', $callback))
        {
            // Return the data as encoded json with a callback
            return $callback.'('.json_encode($data).');';
        }

        // An invalid jsonp callback function provided.
        // Though I don't believe this should be hardcoded here
        $data['warning'] = 'INVALID JSONP CALLBACK: '.$callback;

        return json_encode($data);
    }

    /**
     * Encode data as a serialized array
     *
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @return string Serialized data
     */
    public function to_serialized($data = NULL)
    {
        // If no data is passed as a parameter, then use the data passed
        // via the constructor
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        return serialize($data);
    }

    /**
     * Format data using a PHP structure
     *
     * @param mixed|NULL $data Optional data to pass, so as to override the data passed
     * to the constructor
     * @return mixed String representation of a variable
     */
    public function to_php($data = NULL)
    {
        // If no data is passed as a parameter, then use the data passed
        // via the constructor
        if ($data === NULL && func_num_args() === 0)
        {
            $data = $this->_data;
        }

        return var_export($data, TRUE);
    }

    // INTERNAL FUNCTIONS

    /**
     * @param string $data XML string
     * @return array XML element object; otherwise, empty array
     */
    protected function _from_xml($data)
    {
        return $data ? (array) simplexml_load_string($data, 'SimpleXMLElement', LIBXML_NOCDATA) : [];
    }

    /**
     * @param string $data CSV string
     * @param string $delimiter The optional delimiter parameter sets the field
     * delimiter (one character only). NULL will use the default value (,)
     * @param string $enclosure The optional enclosure parameter sets the field
     * enclosure (one character only). NULL will use the default value (")
     * @return array A multi-dimensional array with the outer array being the number of rows
     * and the inner arrays the individual fields
     */
    protected function _from_csv($data, $delimiter = ',', $enclosure = '"')
    {
        // If NULL, then set as the default delimiter
        if ($delimiter === NULL)
        {
            $delimiter = ',';
        }

        // If NULL, then set as the default enclosure
        if ($enclosure === NULL)
        {
            $enclosure = '"';
        }

        return str_getcsv($data, $delimiter, $enclosure);
    }

    /**
     * @param string $data Encoded json string
     * @return mixed Decoded json string with leading and trailing whitespace removed
     */
    protected function _from_json($data)
    {
        return json_decode(trim($data));
    }

    /**
     * @param string $data Data to unserialize
     * @return mixed Unserialized data
     */
    protected function _from_serialize($data)
    {
        return unserialize(trim($data));
    }

    /**
     * @param string $data Data to trim leading and trailing whitespace
     * @return string Data with leading and trailing whitespace removed
     */
    protected function _from_php($data)
    {
        return trim($data);
    }
    
    /**
     * Export data to a PDF file
     * @package com.tecnick.tcpdf
     * @abstract TCPDF - Example: Default Header and Footer
     *
     * @param array $data
     *
     * Parameters in data variable
     * @param string $filename - Name of the file
     * @param string $content - Contents to be written in the PDF file
     * @param boolean $download - TRUE - Prompt download
     * @param string $path - Path where file to be saved
     */
    public function to_pdf($data = array()) {
        $download = false;
        $path = FCPATH . 'uploads/exports/';
        $filename = 'goclixy';
        $content = '';
        if(isset($data['filename'])) {
            $filename = $data['filename'];
        }
        if(isset($data['content'])) {
            $content = $data['content'];
        }
        if(isset($data['download']) && $data['download']) {
            $download = $data['download'];
        }
        if(isset($data['path'])) {
            $path = $data['path'];
        }
        $this->_CI->load->library('Pdf');
        
        // create new PDF document
        $pdf = new TCPDF(PDF_PAGE_ORIENTATION, PDF_UNIT, PDF_PAGE_FORMAT, true, 'UTF-8', false);
        
        // set document information
        $pdf->SetCreator(PDF_CREATOR);
        $pdf->SetAuthor(settings_item('site.title'));
        $pdf->SetTitle($filename);
        $pdf->SetSubject($filename);
        $pdf->SetKeywords($filename);
        
        // set default header data
        $pdf->SetHeaderData(PDF_HEADER_LOGO, PDF_HEADER_LOGO_WIDTH, '', '', array(0,64,255), array(0,64,128));
        $pdf->setFooterData(array(0,64,0), array(0,64,128));
        
        // set header and footer fonts
        $pdf->setHeaderFont(Array(PDF_FONT_NAME_MAIN, '', PDF_FONT_SIZE_MAIN));
        $pdf->setFooterFont(Array(PDF_FONT_NAME_DATA, '', PDF_FONT_SIZE_DATA));
        
        // set default monospaced font
        $pdf->SetDefaultMonospacedFont(PDF_FONT_MONOSPACED);
        
        // set margins
        $pdf->SetMargins(PDF_MARGIN_LEFT, PDF_MARGIN_TOP, PDF_MARGIN_RIGHT);
        $pdf->SetHeaderMargin(PDF_MARGIN_HEADER);
        $pdf->SetFooterMargin(PDF_MARGIN_FOOTER);
        
        // set auto page breaks
        $pdf->SetAutoPageBreak(TRUE, PDF_MARGIN_BOTTOM);
        
        // set image scale factor
        $pdf->setImageScale(PDF_IMAGE_SCALE_RATIO);
        
        // set some language-dependent strings (optional)
        if (@file_exists(dirname(__FILE__).'/lang/eng.php')) {
            require_once(dirname(__FILE__).'/lang/eng.php');
            $pdf->setLanguageArray($l);
        }
        
        // ---------------------------------------------------------
        
        // set default font subsetting mode
        $pdf->setFontSubsetting(true);
        
        // Set font
        // dejavusans is a UTF-8 Unicode font, if you only need to
        // print standard ASCII chars, you can use core fonts like
        // helvetica or times to reduce file size.
        $pdf->SetFont('dejavusans', '', 12, '', true);
        
        // Add a page
        // This method has several options, check the source code documentation for more information.
        $pdf->AddPage();
        
        // set text shadow effect
        //$pdf->setTextShadow(array('enabled'=>true, 'depth_w'=>0.2, 'depth_h'=>0.2, 'color'=>array(196,196,196), 'opacity'=>1, 'blend_mode'=>'Normal'));
        
        $html = <<<EOD
    			$content
    			
EOD;
        
        // Print text using writeHTMLCell()
        $pdf->writeHTMLCell(0, 0, '', '', $html, 0, 1, 0, true, '', true);
        
        // ---------------------------------------------------------
        
        // Close and output PDF document
        // This method has several options, check the source code documentation for more information.
        // D : send to the browser and force a file download with the name given by name.
        // F : save to a local server file with the name given by name.
        // S : return the document as a string (name is ignored).
        $filename = $filename . '-' . date('Y-m-d');
        $pdf->Output($filename.'.pdf', ($download ? 'D' : 'F'));
    }
    
    /**
     * Export data to a PDF file using DomPDF
     * https://github.com/dompdf/dompdf
     *
     * @param array $data
     *
     * Parameters in data variable
     * @param string $filename - Name of the file
     * @param string $content - Contents to be written in the PDF file
     * @param boolean $download - TRUE - Prompt download
     * @param string $path - Path where file to be saved
     */
    public function to_dompdf($data = array()) {
        $download = false;
        $path = FCPATH . 'uploads/exports/';
        $filename = 'goclixy';
        $content = '';
        if(isset($data['filename'])) {
            $filename = $data['filename'];
        }
        if(isset($data['content'])) {
            $content = $data['content'];
        }
        if(isset($data['download']) && $data['download'] == 1) {
            $download = $data['download'];
        }
        if(isset($data['path'])) {
            $path = $data['path'];
        }
        // instantiate and use the dompdf class
        
        $options = new Options();
        $options->set('defaultFont', 'Helvetica'); // Set Default Font
        $options->set('isRemoteEnabled', TRUE); // Enable to display images from External URLs
        // $options->set('debugKeepTemp', TRUE);
        $options->set('isPhpEnabled', true);
        $options->set('isHtml5ParserEnabled', true); // Enable HTML5Parser
        $dompdf = new Dompdf($options);
        if(isset($data['landscape'])) { // Set Landscape
            $dompdf->set_paper('letter', 'landscape');
        }
        // $dompdf = new Dompdf();
        $dompdf->loadHtml($content);
        
        // Header Settings
        if(isset($data['pagination'])) {
            $w = $dompdf->getCanvas()->get_width();
            $h = $dompdf->getCanvas()->get_height();
            $font = $dompdf->getFontMetrics()->get_font("Helvetica", "bold");
            // change the number 75 to a higher value to move text
            // in case of header pagination, remove the $w- and $h- attributes
            // to change color, change the array RGB values
            $dompdf->getCanvas()->page_text($w-75, $h-28, "{PAGE_NUM} of {PAGE_COUNT}", $font, 12, array(255,255,255));
        }
        
        // (Optional) Setup the paper size and orientation
        // $dompdf->setPaper('A4', 'landscape');
        
        // Header Settings     
        // $font = $dompdf->getFontMetrics()->get_font("Helvetica", "bold");
        // $dompdf->getCanvas()->page_text(72, 18, "Header: {PAGE_NUM} of {PAGE_COUNT}", $font, 10, array(0,0,0));
        
        // Render the HTML as PDF
        $dompdf->render();
        
        // Output the generated PDF to Browser
        if($download) {
            $dompdf->stream($filename . '.pdf');
        } else {
            $output = $dompdf->output();
            file_put_contents($path . '/' . $filename . '.pdf', $output);
        }
    }
    
    /**
     * Export data to an Excel file
     * @param array $data
     *
     * Parameters in data variable
     * @param object $query - Query used to create the Excel File
     * @param array $fields - Field names in the first row, if query object is not passed.
     * @param array $rows - Field data, if query object is not passed.
     * @param string $file_name - Name of the file
     * @param string $file_type - $file_type -- 'Excel2007'
     * @param boolean $archieve - TRUE - create archieve
     * @param boolean $download - TRUE - download file
     */
    public function to_excel($data = array()) {
        $this->_CI->load->library('slug');
        $path = FCPATH . 'uploads/exports/';
        $file_type = 'Xlsx';
        $archieve = false;
        $download = true;
        $wrap_cells = null; // string
        $autosize_columns = array();
        $extra_cells = array();
        if(isset($data['query'])) {
            $query = $data['query'];
            $field_names = $query->list_fields();
            $rows = $query->result();
        }
        if(isset($data['file_name'])) {
            $file_name = $data['file_name'];
        }
        if(isset($data['file_type'])) {
            $file_type = $data['file_type'];
            if($file_type != 'Excel2007') {
                $file_type = 'Xls';
            } else {
                $file_type = 'Xlsx';
            }
        }
        if(isset($data['archieve'])) {
            $archieve = $data['archieve'];
        }
        if(isset($data['download'])) {
            $download = $data['download'];
        }
        if(isset($data['path'])) {
            $path = $data['path'];
        }
        if(isset($data['field_names'])) {
            $field_names = $data['field_names'];
        }
        if(isset($data['rows'])) {
            $rows = $data['rows'];
        }
        if(isset($data['wrap_cells'])) {
            $wrap_cells = $data['wrap_cells'];
        }
        if(isset($data['extra_cells'])) {
            $extra_cells = $data['extra_cells'];
        }
        if(isset($data['autosize_columns'])) {
            $autosize_columns = $data['autosize_columns'];
        }
        
        // Create new spreadsheet object
        $spreadsheet = new Spreadsheet();
        $sheet = $spreadsheet->getActiveSheet();
        
        $objWriter = new Xlsx($spreadsheet);
        
        // Set document properties
        $spreadsheet->getProperties()->setTitle("export")
        ->setCreator(settings_item('site.title'))
        ->setDescription(settings_item('site.title'));
        
        // Add data
        $spreadsheet->setActiveSheetIndex(0);
        
        // Field names in the first row
        $col = 1;
        foreach ($field_names as $field)
        {
            $spreadsheet->getActiveSheet()->setCellValueByColumnAndRow($col, 1, $field);
            $col++;
        }
        
        // Fetching the table data
        $row = 2;
        $rows_count = 1;
        foreach($rows as $data) {
            $col = 1;
            foreach ($data as $field_value) {
                $spreadsheet->getActiveSheet()->setCellValueByColumnAndRow($col, $row, $field_value);
                $col++;
            }
            $rows_count++;
            $row++;
        }
        
        // Set the cell (or group of cells) to enable text wrap in those cells
        if(!empty($wrap_cells)) {
            $spreadsheet->getActiveSheet()->getStyle($wrap_cells)->getAlignment()->setWrapText(true);
        }
        
        // Set the column to a fixed width
        if(!empty($autosize_columns)) {
            foreach($autosize_columns as $columnID) {
                $spreadsheet->getActiveSheet()->getColumnDimension($columnID)->setAutoSize(true);
            }
        } else {
            // Autosize all coulumns
            foreach(range('A',$spreadsheet->getActiveSheet()->getHighestColumn()) as $column) {
                $spreadsheet->getActiveSheet()->getColumnDimension($column)->setAutoSize(true);
            }
        }
        
        // Set Extra Cells
        if(!empty($extra_cells)) {
            foreach($extra_cells as $extra_cell) {
                $spreadsheet->getActiveSheet()->setCellValueByColumnAndRow($extra_cell[0], $extra_cell[1], $extra_cell[2]);
            }
        }
        
        // Set active sheet index to the first sheet, so Excel opens this as the first sheet
        $spreadsheet->setActiveSheetIndex(0);
        $objWriter = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, $file_type);
        
        $filename = ucfirst($file_name). '_'. date('dMy') . '_' . date('H.i.s');
        $filename = $this->_CI->slug->create_slug($filename);
        
        if($file_type == 'Xlsx') {
            $ext = '.xlsx';
        } else {
            $ext = '.xls';
        }
        
        $file_name = $path . $filename . $ext;
        
        //SAVE FILE ONLINE
        $objWriter->save($file_name);
        if($archieve) {
            // create a zip archive of existing file
            $this->_CI->load->library('zip');
            $this->_CI->zip->read_file($file_name);
            
            // write file to the server
            $this->_CI->zip->archive($path . $filename . '.zip');
            
            // delete csv file if user selected zip file
            unlink($path . $filename . $ext);
            
            // Download the file to your desktop. Name it "my_backup.zip"
            if($download) {
                $this->_CI->zip->download($filename.'.zip');
            }
        } else if($download) {
            header('Content-Type: application/vnd.ms-excel'); // Sending headers to force the user to download the file
            header('Content-Disposition: attachment;filename="' . $filename .$ext.'"');
            header('Cache-Control: max-age=0');
            // ISSUE RESOLVE: Excel file cannot open file because the file format or file extension is not valid
            $objWriter = \PhpOffice\PhpSpreadsheet\IOFactory::createWriter($spreadsheet, $file_type);
            //ob_end_clean();
            //ALLOW USER TO DOWNLOAD FILE
            $objWriter->save('php://output');
            exit;
        }
    }
    
    /**
     * Read data from excel for import
     * @param array $data
     */
    public function from_excel($data = array()) {
        $conn = $this->_CI->gc->get_mysqli();
        $file_type = 'Xlsx';        
        $file_name = null;
        if(isset($data['file_type'])) {
            $file_type = $data['file_type'];
            if(strtolower($file_type) != 'excel2007') {
                $file_type = 'Xls';
            }
        }        
        if(isset($data['file_name'])) {
            $file_name = $data['file_name'];
        }
        // check whether file exists or not
        $file_location = FCPATH . 'uploads/imports/'. $file_name;
        if (!file_exists($file_location)) {
            return array('success' => false, 'message' => lang('msg_file_not_exists'));
        }
        
        // Create a new Reader of the type defined in $file_type
        $objReader = \PhpOffice\PhpSpreadsheet\IOFactory::createReader($file_type);
        
        // Advise the Reader that we only want to load cell data
        $objReader->setReadDataOnly(true);
        
        // If user selected XLSX as the file type but uploaded an XLS file and vice versa
        $file_location = str_replace('\\', '/', $file_location);
        if (!$objReader->canRead($file_location)) {
            return array('success' => false, 'message' => lang('error_file_format'));
        }
        
        // Load $file_location to a Spreadsheet Object
        $spreadsheet = $objReader->load($file_location);
        
        // read active sheet
        $worksheet = $spreadsheet->getActiveSheet();
        
        // Store data from the activeSheet to the varibale in the form of Array
        $data = array(1, $worksheet->toArray(null,true,true,true)); 
        
        //get only the Cell Collection
        $cell_collection = $spreadsheet->getActiveSheet()->getCoordinates();
        
        //extract to a PHP readable array format
        foreach ($cell_collection as $cell) {
            $column = $spreadsheet->getActiveSheet()->getCell($cell)->getColumn();
            $row = $spreadsheet->getActiveSheet()->getCell($cell)->getRow();
            $data_value = $spreadsheet->getActiveSheet()->getCell($cell)->getValue();
            // header should be in row 1 only. of course this can be modified to suit your need.
            if ($row == 1) {
                $header[$column] = $data_value; //SAVE HEADER VALUES
            } else {
                $arr_data[$row][$header[$column]] = mysqli_real_escape_string($conn, trim($data_value)); //IMPORTANT OTHERWISE BATCH_INSERT WILL NOT WORK
            }
        }
        if(!empty($header) && !empty($arr_data)) {
            return array('success' => true, 'header' => $header, 'data' => $arr_data);
        } else {
            return array('success' => false, 'message' => lang('msg_nothing_to_update'));
        }
    }
}
