<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Translations Library
 *
 * Provides methods to retrieve translations from database tables.
 *
 * @uses Bonfire\Libraries\Curl (CodeIgniter cURL library from Phil Sturgeon)
 *
 * @package Application\Libraries\Translations
 * @author  GoClixy Team
 * @link    https://docs.goclixy.com
 * @todo    Update Link to a Docs/Guides on the translations methods
 */
class Translations
{
    /** @var string separator used in translate input field. */
    private $separator = '_trlng_';
    
    /** @var string translations table. */
    private $table = 'translations';
    
    public $translated_fields = array(
        'categories' => array('name', 'slug', 'description', 'meta_title', 'meta_description', 'meta_keywords'),
        'pages' => array('title', 'slug', 'body', 'meta_title', 'meta_description', 'meta_keywords'),
        'news' => array('title', 'slug', 'description', 'meta_title', 'meta_description', 'meta_keywords'),
        'testimonials' => array('description'),
        'packages' => array('title', 'description'),
        'module_email_templates' => array('subject', 'message'),
        'ad_packages' => array('title', 'description'),
        'tags' => array('name'),
        'settings' => array(
            'site.title',
            'site.footer_text',
            'site.meta_title',
            'site.meta_keywords',
            'site.meta_description',
            'seo.search_meta_title',
            'seo.search_meta_keywords',
            'seo.search_meta_description',
            'seo.page_meta_title',
            'seo.page_meta_keywords',
            'seo.page_meta_description',
            'seo.location_meta_title',
            'seo.location_meta_keywords',
            'seo.location_meta_description',
            'seo.listing_meta_title',
            'seo.listing_meta_keywords',
            'seo.listing_meta_description',
            'seo.category_meta_title',
            'seo.category_meta_keywords',
            'seo.category_meta_description'
        )
    );
    
    /** @var object A pointer to the CodeIgniter instance. */
    protected $ci;
    
    /** @var object A point to the Zend Cache. */
    protected $zt_cache;
    
    // ------------------------------------------------------------------------
    
    /**
     * The Translations Construct allows optionally passing settings in array and stores them
     * 
     * @return void
     */
    public function __construct($options = array())
    {
        $this->ci =& get_instance();
        
        // Load Cache Instance
        $this->ci->load->library('slug');
        $this->ci->load->library('gzf_cache', array('lifetime' => 900));
        $this->zt_cache = $this->ci->gzf_cache->get_instance();
        
        $this->ci->load->helper('gc');
        $module_fields = $this->module_fields();
        if(!empty($module_fields)) {
            $this->translated_fields = array_merge($this->translated_fields, $module_fields);
        }        
    }
    
    /**
     * Get Admin Enabled languages
     * @return string
     */
    public function field_languages() {
        $this->ci->load->library('gzf_cache', array('lifetime' => 900));
        $this->zt_cache = $this->ci->gzf_cache->get_instance();
        $cacheID = 'field_languages';
        if(!$field_languages = $this->zt_cache->load($cacheID)) {
            $field_languages = $this->process_languages();
            $this->zt_cache->save($field_languages, $cacheID);
        }
        if(empty($field_languages)) { // No language is active
            redirect(site_url());
        }
        $this->ci->session->set_userdata('field_languages', $field_languages);
        return $field_languages;
    }
    
    /**
     * Display language dropdown with Input Field
     * @return string[]|NULL[][]|boolean
     */
    private function process_languages() {
        $languages = array();
        if($this->ci->session->userdata('language')) {
            // override the default language with the user selected language
            $default_language = $this->ci->session->userdata('language');
        } else {
            $default_language = settings_item('site.default_language');
        }
        $setting_languages = unserialize(settings_item('site.languages'));
        if(!in_array($default_language, $setting_languages)) {
            $setting_languages[] = $default_language;
        }
        $default_language = strtolower($default_language);
        if(!empty($setting_languages)) {
            $result = $this->ci->db->select('code, label, name, alpha_2')->where('label IN ("' . implode('", "', $setting_languages) . '")')->get('languages')->result();
            if($result) {
                foreach($result as $language) {
                    $languages[] = array($language->code, $language->name, $language->alpha_2);
                    // get code of the default language
                    if(strtolower($language->label) == $default_language) {
                        $default_language_code = $language->code;
                    }
                }
                // if the default language is not in the list, set default language to english.
                if(!isset($default_language_code)) {
                    $default_language_code = 'en';
                }
                if(!empty($languages)) {
                    $language_options = [];
                    foreach($languages as $lang) {
                        $key = $lang[0];
                        $value = $lang[1];
                        $language_options[] = <<<EOL
                        
            <a href="javascript:toggleTranslate('{$key}');" tabindex="-1" class="dropdown-item">{$value}</a>
EOL;
                    }
                    $language_options = implode("", $language_options);
                    $dropdown_menu = <<<EOL
<div class="dropdown-menu">{$language_options}
        </div>
EOL;
                    return array('default' => $default_language_code, 'languages' => $languages, 'menu' => $dropdown_menu);
                }
            }
        }
        return false;
    }
    
    /**
     * Function called before translations
     *
     * Prepare fields for save, also get the default field values for language fallback
     * e.g. if a user selected FRENCH as the default language, but there are no translations for FRENCH
     * then show data using the default values stored in the module table.
     */
    public function validate_prepare_fields($user_id = 0, $module = null, $role = 'admin') {
        if(settings_item('site.multi_language') == 0) {
            // Atleast one rule must exist to validate the validations,
            // hence a dummy rule is added to fulfil this condition
            $this->ci->form_validation->set_rules('dummy', 'lang:label_title', 'trim');
            return true; // do not proceed further if the multi language feature is disabled on the site
        }
        
        $default_language_code = settings_item('site.language_code');
        $translate_fields = array();
        $translate_unset = array();
        $fields = array();
        foreach($_POST as $key => $value) {
            if(!empty($value)) { // Do not process blank fields
                // A transable field
                if (strpos($key, $this->separator) !== false) {
                    $field = explode($this->separator, $key);
                    if(!empty($field[0]) && strlen($field[1]) == 2) {
                        
                        // Set validation rules
                        $this->ci->form_validation->set_rules($key, 'lang:label_' . $field[0], 'trim');
                        
                        // a user may have kept the default language input field blank e.g. title_eng
                        // in this case use any one of the field having a value
                        // store field name in an array, fill it if the default language field has a value
                        if(empty($fields[$field[0]])) {
                            $fields[$field[0]] = $value; // store a value and compare it with $_POST variable if the default language input field is kept blank
                        }
                        
                        // set the default field post value for fallback
                        if($field[1] == $default_language_code) {
                            $_POST[$field[0]] = $value;
                            $fields[$field[0]] = $value; // field name stored, do not look further
                        }
                        
                        // store field
                        $translate_fields[] = array(
                            'language_code' => $field[1],
                            'name' => $field[0],
                            'value' => $value
                        );
                    }
                    // Do not unset POST fields here otherwise data will be gone on validation error
                    $translate_unset[] = $key;
                    // unset($_POST[$key]); // Unset the variable
                }
            }
        }
        
        // compare stored fields with POST variable
        // if a key does not exist in the POST variable, add it from the fields variable
        if(!empty($fields)) {
            foreach($fields as $post_key => $post_value) {
                if(!isset($_POST[$post_key])) {
                    $_POST[$post_key] = $post_value;
                }
            }
        }
        
        // Store fields in a session variable for SAVE
        // Session is creating issues with large data, e.g. pages have lots of information in the description field.
        // i.e. zend cache is used
        // $this->ci->session->set_flashdata('translate_unset', $translate_unset);
        // $this->ci->session->set_flashdata('translate_fields', $translate_fields);
        if(!empty($user_id) && !empty($module) && !empty($role)) {
            $cache_id = $user_id . '_' . $module . '_' . $role;
            $data = array('translate_unset' => $translate_unset, 'translate_fields' => $translate_fields);
            $this->zt_cache->save($data, 'translate_' . $cache_id, array($cache_id));
        } else {
            $this->ci->session->set_flashdata('translate_unset', $translate_unset);
            $this->ci->session->set_flashdata('translate_fields', $translate_fields);            
        }
    }
    
    /**
     * Unset unwanted POST fields only after validation success but before data preparation
     */
    public function translate_unset($user_id = 0, $module = null, $role = 'admin') {
        $translate_unset = null;
        if(!empty($user_id) && !empty($module) && !empty($role)) {
            $cache_id = 'translate_' . $user_id . '_' . $module . '_' . $role;
            $data = $this->zt_cache->load($cache_id);
            if(!empty($data['translate_unset'])) {
                $translate_unset = $data['translate_unset'];
            }
        } else {
            $translate_unset = $this->ci->session->flashdata('translate_unset');
        }
        if(!empty($translate_unset)) {
            foreach($translate_unset as $key) {
                unset($_POST[$key]); // Unset the variable
            }
        }
    }
    
    /**
     * Save data only after validation success
     * @param string $module - Name of the Module
     * @param number $row_id - ID of the row
     * @param number $insert_id - ID of the inserted row - in case of INSERT
     */
    public function save_translations($module, $row_id, $insert_id, $user_id = 0, $role = 'admin') {
        if(!$insert_id) {
            return true;
        }
        if($row_id == 0) {
            $row_id = (int) $insert_id;
        }
        if($row_id == 0) {
            return true;
        }
        
        if(settings_item('site.multi_language') == 0) {
            return true; // do not proceed further if the multi language feature is disabled on the site
        }
        $module_id = gc_module_id($module);
        $translate_fields = null;
        if(!empty($user_id) && !empty($module) && !empty($role)) {
            $cache_id = 'translate_' . $user_id . '_' . $module . '_' . $role;
            $data = $this->zt_cache->load($cache_id);
            if(!empty($data['translate_fields'])) {
                $translate_fields = $data['translate_fields'];
            }
        } else {
            $translate_fields = $this->ci->session->flashdata('translate_fields');
        }
        $multi_language_data = array();
        if($translate_fields) {
            foreach($translate_fields as $translate_field) {
                // Append prefix with each settings table field e.g. site.title
                if($module == 'settings') {
                    foreach($this->translated_fields[$module] as $setting_field) {
                        $setting_field_array = explode('.', $setting_field); // Get field name
                        if($setting_field_array[1] == $translate_field['name']) {
                            $translate_field['name'] = $setting_field_array[0] . '.' . $translate_field['name'];
                        }
                    }
                }
                // prepare data to be stored in the multi language table
                $multi_language_data[] = array(
                    'language_code' => $translate_field['language_code'],
                    'module_id' => $module_id,
                    'row_id' => $row_id,
                    'name' => $translate_field['name'],
                    'value' => $translate_field['name'] == 'slug' ? $this->ci->slug->create_slug($translate_field['value']) : $translate_field['value']
                );
            }
        }
        if(!empty($multi_language_data)) {
            // perhaps, the user has deleted a field, so purge existing data first
            $this->ci->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->delete($this->table);
            
            // insert new data
            $this->ci->db->insert_batch($this->table, $multi_language_data);
        }
    }
    
    /**
     * Get translations for EDIT and DETAIL pages
     * @param string $module
     * @param number $row_id
     * @param array $record
     */
    public function get_translations($module, $row_id, $record, $type_id = 0, $language = null, $translated_fields = array()) {
        if(!empty($translated_fields)) {
            $this->translated_fields[$module] = $translated_fields;
        }
        if(settings_item('site.multi_language') == 0) {
            return $record; // do not proceed further if the multi language feature is disabled on the site
        }
        if(!isset($this->translated_fields[$module])) {
            return $record;
        }
        $where = array('module_id' => gc_module_id($module), 'row_id' => $row_id);
        if($type_id == 1) { // Detail Page
            $language = $this->get_language_code(!empty($language) ? $language : $this->ci->session->userdata('language'));
            $where['language_code'] = $language; // Get specific language record
        }
        $result = $this->ci->db->get_where($this->table, $where)->result();
        if($result) {
            if($type_id == 0) { // Edit Page
                $fields_without_translations = $this->translated_fields[$module];
                foreach($result as $row) {
                    // unset all fields having value in the translations table
                    if(in_array($row->name, $fields_without_translations)) {
                        $field_key = array_search($row->name, $fields_without_translations);
                        unset($fields_without_translations[$field_key]);
                    }
                    $field_name = $row->name . $this->separator . $row->language_code;
                    $record->{$field_name} = $row->value;
                }
                // fields having no value in the translations table
                // display the default value to the user
                if(!empty($fields_without_translations)) {
                    $record = $this->non_translated_fields($fields_without_translations, $record);
                }
            } else { // Detail Page
                foreach($record as $key => $value) {
                    foreach($result as $row) {
                        if($key == $row->name) {
                            $record->{$key} = $row->value;
                        }
                    }
                }
            }
        } else if($type_id == 0) { // Edit Page - Set the default translations
            $record = $this->non_translated_fields($this->translated_fields[$module], $record);
        }
        return $record;
    }
    
    /**
     * Display default value for all non translated fields
     * @param array $fields
     * @param array $record
     * @return array
     */
    private function non_translated_fields($fields, $record) {
        foreach($fields as $field) {
            if(isset($record->{$field})) {
                $field_name = $field . $this->separator . settings_item('site.language_code');
                $record->{$field_name} = $record->{$field};
            }
        }
        return $record;
    }
    
    /**
     * Purge Translations on record delete
     * @param string $module
     * @param number $row_id
     */
    public function purge_tanslations($module, $row_id) {
        $module_id = gc_module_id($module);
        $this->ci->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->delete($this->table);
    }
    
    /**
     * Get Language Code
     * @param string $language
     * @param string $return
     * @return string
     */
    public function get_language_code($language, $return = 'en') {
        $code = $this->ci->db->select('code')->get_where('languages', array('label' => $language))->row();
        if($code) {
            $return = $code->code;
        }
        return $return;
    }
    
    /**
     * Get selected/default language global settings from the translations table
     * @param string $settings
     * @return array
     */
    public function site_settings($records) {
        if(settings_item('site.multi_language') == 0) {
            return $records; // do not proceed further if the multi language feature is disabled on the site
        }
        $this->ci->load->library('session');
        $language = ($this->ci->session->userdata('language') ? $this->ci->session->userdata('language') : settings_item('site.default_language'));
        $cacheID = 'settings_' . str_replace('-', '_', $this->ci->slug->create_slug($language));
        if(!$settings = $this->zt_cache->load($cacheID)) {
            $language = $this->get_language_code($language);
            $result = $this->ci->db->where(array('module_id' => 35, 'row_id' => 1, 'language_code' => $language))->get($this->table)->result();
            if($result) {
                foreach($records as $key => $setting) {
                    foreach($result as $row) {
                        if($setting->name == $row->name) {
                            $records[$key]->value = $row->value;
                        }
                    }
                }
            }
            $this->zt_cache->save($records, $cacheID, array('site_settings'));
        }
        return !empty($settings) ? $settings: $records;
    }
    
    /**
     * Get Fields by Key
     *
     * e.g. Multi Language - Get all fields where lang is 1
     *
     * @param string $module
     */
    public function module_fields($module = 'all', $key = 'multi_language', $value = 1) {
        $cacheID = 'module_fields_' . $module . '_' . $key . '_' . $value;
        $field_names = array();
        if(!$field_names = $this->zt_cache->load($cacheID)) {
            $this->ci->load->model('fields/fields_model');
            $field_names = $this->ci->fields_model->get_fields_by_key($module, $key, $value);
            $this->zt_cache->save($field_names, $cacheID, array('module_fields_by_key'));
        }
        return $field_names;
    }
    
    /**
     * Front Controller - Language Change Handling
     * @param string $language
     */
    public function language_change($language) {
        if(!$this->ci->session->userdata('language_code')) {
            // Check whether the language is RTL or not
            $selected_language = $this->ci->db->select('code, is_rtl')->get_where('languages', array('label' => $language))->row();
            if($selected_language && $selected_language->is_rtl == 1) {
                // if RTL, set the is_rtl session variable to load the rtl css files
                $this->ci->session->set_userdata('is_rtl', 1);
            } else {
                $this->ci->session->unset_userdata('is_rtl');
            }
            $language_code = (!empty($selected_language->code) ? $selected_language->code : settings_item('site.language_code'));
            $this->ci->session->set_userdata('language_code', $language_code);
        }
    }
    
    /**
     * Display languages from the languages table.
     * Load all languages from the languages table
     * @param boolean $flip
     * @param string $code - Get language name from code
     * @param string $name - Get code from the language name
     * @return NULL[]
     */
    public function get_languages($flip = true, $code = null, $name = null) {
        $cacheID = 'site_all_languages_' . $flip;
        if(!$languages = $this->zt_cache->load($cacheID)) {
            $this->ci->load->model('languages/languages_model');
            $records = $this->ci->languages_model->select('code, label')->order_by('label')->find_all();
            $languages = array();
            if($records) {
                if($flip) {
                    foreach($records as $record) {
                        $languages[$record->code] = $record->label;
                    }
                } else {
                    foreach($records as $record) {
                        $languages[$record->label] = $record->code;
                    }
                }
            }
            $this->zt_cache->save($languages, $cacheID, array('languages'));
        }
        if(!empty($code)) {
            return isset($languages[$code]) ? $languages[$code] : null;
        } else if(!empty($name)) {
            return array_search(strtolower($name), $languages);
        } else {
            return $languages;
        }
    }
    
    /**
     * Common function to fetch language related records
     * @param string $module
     * @param array $table_fields
     * @param string $where
     * @param string $order_by
     * @param number $limit
     */
    public function get_translated_rows($module, $table_fields = array(), $where = NULL, $order_by = NULL, $limit = 10, $select_extra = NULL, $join_extra = NULL, $params = array()) {        
        $query = $this->build_query($module, $table_fields, (!empty($params['translated_fields']) ? $params['translated_fields'] : array()));
        $sql = $query['select'];        
        if(!empty($select_extra)) {
            $sql .= ', ' . $select_extra;
        }
        $sql .= " FROM " . $this->ci->db->dbprefix . $module . " M";
        if(!empty($query['join'])) {
            $sql .= $query['join'];
        }
        if(!empty($join_extra)) {
            $sql .= PHP_EOL . $join_extra;
        }
        if(!empty($where)) {
            $sql .= " WHERE " . $where;
        }
        $sql .= " ORDER BY " . $order_by;
        $sql .= " LIMIT " . $limit;
        $result = $this->ci->db->query($sql)->result();
        return $result;
    }
    
    /**
     * Build Multilanguage query for different modules
     * @param string $module
     * @param array $table_fields - Alway pass table fields for query build
     * @param array $translated_fields - Restrict number of fields to be JOINED
     * e.g. Category sub query get Name only
     */
    public function build_query($module, $table_fields = array(), $translated_fields = array(), $table_alias = 'M.') {
        if(!isset($this->translated_fields[$module])) {
            $select_statement = "SELECT " . $this->list_columns($module, $table_fields, false, $translated_fields, $table_alias);
            $join_statement = NULL;
        } else {
            if(empty($translated_fields)) {
                $translated_fields = $this->translated_fields[$module];
            }
            $module_id = gc_module_id($module);
            if(settings_item('site.multi_language') == 0) {
                $select_statement = "SELECT " . $this->list_columns($module, $table_fields, false, $translated_fields, $table_alias);
                $join_statement = NULL;
            } else if(isset($this->translated_fields[$module])) {
                /**
                 * SELECT statement does not overwrite fields
                 * so we need to find and remove them to avoid column name duplication
                 */
                $language_code = $this->ci->session->userdata('language_code');
                $select_fields = array();
                $join_fields = array();
                foreach($translated_fields as $translated_field) {
                    $select_fields[] = "(CASE WHEN TRS." . $translated_field . " IS NULL THEN " . $table_alias . $translated_field . " ELSE TRS." . $translated_field . " END) AS " . $translated_field . PHP_EOL;
                    $join_fields[] = "MAX(IF(name = '" . $translated_field . "', value, NULL)) AS " . $translated_field . PHP_EOL;
                }
                $select_fields = implode(', ', $select_fields);
                $join_fields = implode(', ', $join_fields);
                $select_statement = "SELECT " . $this->list_columns($module, $table_fields, true, $translated_fields, $table_alias) . ", " . PHP_EOL . $select_fields;
                $join_statement = " LEFT JOIN (SELECT row_id, module_id, " . $join_fields . " FROM " . $this->ci->db->dbprefix . "translations WHERE language_code = '" . $language_code . "' AND module_id = " . $module_id . " GROUP BY row_id) AS TRS ON TRS.row_id = " . $table_alias . "id";
            }
        }
        return array('select' => $select_statement, 'join' => $join_statement);
    }
    
    /**
     * SELECT statement does not overwrite same column
     * so we need to find and remove them to avoid column name duplication
     * @param string $module
     * @param array $table_fields
     * @param boolean $multi_language
     * $param array $translated_fields
     * @return array
     */
    private function list_columns($module, $table_fields, $multi_language, $translated_fields, $table_alias) {
        $return_string = true;
        if(empty($table_fields)) {
            $table_fields = $this->ci->db->list_fields($module);
        }
        $fields = $table_fields;
        if($multi_language) {
            $fields = array_diff($table_fields, $translated_fields);
            if(empty($fields)) {
                $fields = $table_fields;
            }
        }
        
        $select_fields = array();
        foreach($fields as $field) {
            $select_fields[] = $table_alias . $field;
        }
        if($return_string) {
            return implode(', ', $select_fields);
        } else {
            return $select_fields;
        }
    }
    
    /**
     * Listings, Categories and Tags Table Repeated Joins
     * @return string
     */
    public function common_joins($module) {
        switch($module) {
            case 'listings':
                $table_fields = array('id', 'logo', 'title', 'slug', 'city_id', 'city', 'deleted', 'status');
                $translated_fields = array('title', 'slug');
                break;
            case 'categories':
                $table_fields = array('id', 'slug', 'parent_id', 'name', 'active', 'counts');
                $translated_fields = array('name');
                break;
            case 'tags':
                $table_fields = array('id', 'name', 'active');
                $translated_fields = array('name');
                break;
        }
        // Get Translated Fields
        $query_join = $this->build_query($module, $table_fields, $translated_fields);
        $sql = " LEFT JOIN ( " . PHP_EOL;
        $sql .= $query_join['select'];
        $sql .= " FROM " . $this->ci->db->dbprefix . $module . " M" . PHP_EOL;
        if(!empty($query_join['join'])) {
            $sql .= $query_join['join'];
        }
        $sql .= ")";
        return $sql;
    }
    
    /**
     * Array to Attributes e.g. array('created_on' => 'DESC') to created_on DESC
     * @param array $array
     * @return NULL|string
     */
    private function array_to_attributes($array) {
        $attributes_str = NULL;
        foreach ($array as $attribute => $value) {
            $attributes_str .= 'M.' . $attribute . ' ' . $value;
        }
        return $attributes_str;
    }
    
    /**
     * Bulk Import Translations
     * @param string $module
     * @param integer $row_id
     * @param array $fields
     */
    public function bulk_import($module, $row_id, $fields) {
        $module_id = gc_module_id($module);
        if(!empty($fields)) {
            $data = array();
            foreach($fields as $key => $value) {
                $str = str_replace('lang_', '', $key);
                $arr = explode("_", $str, 2);
                $language_code = $arr[0];
                $field_name = $arr[1];
                // import field only if it exists in the translated fields array
                if(isset($this->translated_fields[$module]) && in_array($field_name, $this->translated_fields[$module])) {
                    $data[] = array(
                        'language_code' => $language_code,
                        'module_id' => $module_id,
                        'row_id' => $row_id,
                        'name' => $field_name,
                        'value' => $value
                    );
                }
            }
            if(!empty($data)) {
                // perhaps, the user has deleted a field, so purge existing data first
                $this->ci->db->where(array('module_id' => $module_id, 'row_id' => $row_id))->delete($this->table);
                
                // insert new data
                $this->ci->db->insert_batch($this->table, $data);
            }
        }
    }
    
    /**
     * Set Language in the FrontController on change and page load
     * @param string $language
     */
    public function set_language($language) {
        $language = strtolower($language);
        $this->ci->session->set_userdata('language', strtolower($language));
        
        // Save language code and RTL value in a session variable
        $this->ci->session->unset_userdata('language_code');
        $this->language_change($language);
    }
    
    /**
     * Get enabled languages from the Settings
     */
    public function site_languages() { 
        // retrieve allowed languages from database and pass them as a paramter
        if(!$this->ci->session->userdata('site_languages')) {
            $cacheID = 'site_languages';
            if(!$languages = $this->zt_cache->load($cacheID)) {
                $languages = $this->ci->db->select('code, name, label, alpha_2 AS flag')->where("label IN ('" . implode("','", unserialize(settings_item('site.languages'))) . "')")->get('languages')->result();
                $this->zt_cache->save($languages, $cacheID, array('languages'));
            }
            $this->ci->session->set_userdata('site_languages', $languages);
        }
    }
}