<?php defined('BASEPATH') || exit('No direct script access allowed');
/**
 * Bonfire
 *
 * An open source project to allow developers to jumpstart their development of
 * CodeIgniter applications.
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2015, Bonfire Dev Team
 * @license   http://opensource.org/licenses/MIT The MIT License
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

/**
 * Emailer Settings Context
 *
 * Allows the management of the Emailer. Assists in setting up the proper email
 * settings, as well as editing the template and viewing emails in the queue.
 *
 * @package Bonfire\Modules\Emailer\Controllers\Components
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs
 */
class Components extends Admin_Controller
{
    protected $permissionCreate = 'Bonfire.Emailer.Manage';
    protected $permissionDelete = 'Bonfire.Emailer.Manage';
    protected $permissionEdit   = 'Bonfire.Emailer.Manage';
    protected $permissionView   = 'Bonfire.Emailer.Manage';
    protected $navUrl = '/components/emailer';
    protected $table_subscribers = 'subscriber_users';
    
    /**
     * Set up the permissions and load the language file.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();

        $this->auth->restrict('Site.Settings.View');
        $this->auth->restrict('Bonfire.Emailer.Manage');

        $this->lang->load('emailer');
        $this->load->library('gc');
        $this->load->model('subscribers/subscribers_model');

        Assets::add_module_js('emailer', 'components.js');
        Template::set_block('sub_nav', 'components/_sub_nav');
    }

    /**
     * Displays the emailer settings
     *
     * @return void
     */
    public function index()
    {
        $this->auth->restrict('Emailer.Components.Settings');
        gc_demo_mode(array('redirect' => 'emailer'));
        Assets::add_js('bootstrap-show-password.min.js');
        if (isset($_POST['save'])) {
            $this->form_validation->set_rules('sender_email', 'lang:emailer_system_email', 'required|trim|valid_email|max_length[120]');
            $this->form_validation->set_rules('protocol', 'lang:emailer_email_server', 'trim');

            $protocol = $this->input->post('protocol');
            if ($protocol == 'sendmail') {
                $this->form_validation->set_rules('mailpath', 'lang:emailer_sendmail_path', 'required|trim');
            } elseif ($protocol == 'smtp') {
                $this->form_validation->set_rules('smtp_host', 'lang:emailer_smtp_address', 'required|trim');
                $this->form_validation->set_rules('smtp_crypto', 'lang:emailer_smtp_crypto', 'trim');
                $this->form_validation->set_rules('smtp_user', 'lang:emailer_smtp_username', 'trim');
                $this->form_validation->set_rules('smtp_pass', 'lang:emailer_smtp_password', 'trim'); // |matches_pattern[[A-Za-z0-9!@#\%$^&+=]{2,20}]
                $this->form_validation->set_rules('smtp_port', 'lang:emailer_smtp_port', 'trim|numeric');
                $this->form_validation->set_rules('smtp_timeout', 'lang:emailer_smtp_timeout', 'trim|numeric');
            }

            if ($this->form_validation->run() === false) {
                Template::set_message(lang('emailer_settings_save_error'), 'danger');
            } else {
                $data = array(
                    array('name' => 'sender_email', 'value' => $this->input->post('sender_email')),
                    array('name' => 'mailtype',     'value' => $this->input->post('mailtype')),
                    array('name' => 'protocol',     'value' => $protocol),
                    array('name' => 'mailpath',     'value' => $_POST['mailpath']),
                    array('name' => 'smtp_host',    'value' => isset($_POST['smtp_host']) ? $_POST['smtp_host'] : ''),
                    array('name' => 'smtp_crypto',  'value' => isset($_POST['smtp_crypto']) ? $_POST['smtp_crypto'] : ''),
                    array('name' => 'smtp_user',    'value' => isset($_POST['smtp_user']) ? $_POST['smtp_user'] : ''),
                    array('name' => 'smtp_pass',    'value' => isset($_POST['smtp_pass']) ? $_POST['smtp_pass'] : ''),
                    array('name' => 'smtp_port',    'value' => isset($_POST['smtp_port']) ? $_POST['smtp_port'] : ''),
                    array('name' => 'smtp_timeout', 'value' => isset($_POST['smtp_timeout']) ? $_POST['smtp_timeout'] : '5'),
                );

                // Save the settings to the db
                $updated = $this->settings_model->update_batch($data, 'name');
                if ($updated) {
                    // Success, reload the page so they can see their settings
                    Template::set_message(lang('emailer_settings_save_success'), 'success');
                    redirect(SITE_AREA . $this->navUrl);
                }

                Template::set_message(lang('emailer_settings_save_error'), 'danger');
            }
        }

        // Load our current settings
        $settings = $this->settings_model->select('name,value')
                                         ->find_all_by('module', 'email');

        Template::set($settings);
        Template::set('toolbar_title', lang('emailer_settings'));

        Template::render();
    }
    
    /**
     * Dashboard Summary Redirect
     * @param string $module
     */
    /*public function manage($module, $type = null) {
        switch($module) {
            case 'history':
                Template::redirect(site_url(SITE_AREA . '/components/emailer/view/' . $module . '/' . $type));
                break;
            default:
                Template::redirect(site_url(SITE_AREA . '/components/emailer/queue'));
        }
    }*/
    
    /**
     * Manage News
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    public function manage($module = 'templates', $filter = 'all', $offset = 0) {
        $extra = array(
            'model' => 'emailer',
            'context' => 'components',
            'fields' => $module,
            'prefix' => 'email'
        );
        if($module != 'sender_history') {
            $extra['actions'] = array( // Extra buttons
                array('type' => 'link', 'link' => site_url(SITE_AREA . '/components/emailer/form/' . $module . '/edit/%s'), 'label' => '<i class="fas fa-edit fa-fw mr-1"></i>' . lang('edit'), 'class' => 'btn btn-xs btn-primary'),
            );
        }
        if($module == 'templates') {
            $extra['advanced_filter'] = array('group_ids');
            $extra['extra_action'] = anchor(site_url(SITE_AREA . '/components/emailer/manage/template_groups'), '<i class="fas fa-object-group fa-fw mr-1"></i>' . lang('gc_menu_groups'), 'class="btn btn-success btn-sm ml-1"');
        } else if($module == 'template_groups') {
            Template::set('extra_action', anchor(site_url(SITE_AREA . '/components/emailer/manage/templates'), '<i class="fas fa-arrow-circle-left fa-fw mr-1"></i>' . lang('email_templates'), 'class="btn btn-danger btn-sm ml-1"'));
        } else if($module == 'sender_history') {
            if(isset($_POST['clear_sender_id'])) {
                $this->session->unset_userdata('emailer_history_sender_id');
            }
            $sender_id = (int) $this->session->userdata('emailer_history_sender_id');
            $extra_actions = '';
            if(!empty($sender_id)) {
                $extra['query_where'] = 'M.sender_id = ' . $sender_id;
                $extra_actions = '<button name="clear_sender_id" class="btn btn-danger btn-xs"><i class="fas fa-times fa-fw mr-1"></i>' . lang('bf_clear') . ' ' . lang('singular_senders') . ' (' . $sender_id . ')</button>';
                $this->db->where('sender_id', $sender_id);
            }
            $row = $this->db->select('SUM(failure_counts) AS failure_counts, SUM(success_counts) AS success_counts')->get('email_sender_history')->row();
            if($row) {
                $extra_actions .= '<span class="ml-2">' . lang('label_total') . '</span>:<span class="btn btn-success btn-sm mr-2 ml-2">' . (int) $row->success_counts . '</span><span class="btn btn-danger btn-sm">' . (int) $row->failure_counts . '</span>';
            }
            $extra['advanced_filter'] = array('sender_id', 'date_from');
            $extra['advanced_filter_date'] = true;
            $extra['extra_action'] = $extra_actions;
            // Template::set('extra_action', $extra_actions);
        }
        $this->gc->module_view($module, $filter, $offset, 'admin', $extra);
    }
    
    /**
     * Add/Edit News Data
     *
     * @param string $module
     * @param string $action
     *
     * @return void
     */
    public function form($module = 'templates', $action = 'create') {
        /*Assets::add_js(array(
            'tinymce/tinymce.min.js',
        ));*/
        $params = array(
            'model' => 'emailer',
            'context' => 'components',
            'fields_method' => $module,
            'prefix' => 'email',
            'redirect_url' => site_url(SITE_AREA . '/components/emailer/manage/' . $module)
        );
        $this->gcrud->form($module, $action, 'admin', $params);
    }

    /**
     * Display the screen to edit the email templagtes
     *
     * @return void
     */
    public function template() {
        $this->auth->restrict('Email_Template.Components.View');
        Assets::add_js(array(
            'tinymce/tinymce.min.js',
        ));
        
        Assets::add_module_js('emailer', 'emailer.js');
        
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $header = $_POST['header'];
            $footer = $_POST['footer'];
            
            $this->load->helper('file');
            
            write_file(APPPATH . 'modules/emailer/views/email/_header.php', $header, 'w+');
            write_file(APPPATH . 'modules/emailer/views/email/_footer.php', $footer, 'w+');
            
            Template::set_message(sprintf(lang('msg_save_success'), lang('em_template')), 'success');
            
            redirect(SITE_AREA . $this->navUrl . '/template');
        }
        
        Template::set('toolbar_title', lang('emailer_email_template'));
        
        Template::render();
    }

    /**
     * Send a test email
     *
     * @return void
     */
    public function test()
    {
        if (! isset($_POST['test'])) {
            gc_demo_mode();
            $this->security->csrf_show_error();
        }

        $this->load->library('emailer');
        $this->emailer->enable_debug(true);

        $data = array(
            'to'      => $this->input->post('email'),
            'subject' => lang('emailer_test_mail_subject'),
            'message' => lang('emailer_test_mail_body'),
         );

        $success = $this->emailer->send($data, false);

        Template::set('toolbar_title', lang('emailer_email_test'));
        Template::set('success', $success);
        Template::set('debug', $this->emailer->debug_message);

        Template::render();
    }

    /**
     * Display all of the emails currently in the queue.
     *
     * @return void
     */
    public function queue($filter = 'all', $offset = 0) {
        $this->auth->restrict('Emailer.Queue.View');
        $this->load->library('pagination');
        $this->load->model('emailer/emailer_model');
        
        // Deleting anything?
        if (isset($_POST['delete'])) {
            $checked = $this->input->post('checked');
            if (! empty($checked) && is_array($checked)) {
                $result = true;
                $emailError = '';
                foreach ($checked as $pid) {
                    $deleted = $this->emailer_model->delete($pid);
                    if (! $deleted) {
                        $result = false;
                        $emailError = $this->emailer_model->error;
                    }
                }
                
                if ($result) {
                    Template::set_message(sprintf(lang('emailer_delete_success'), count($checked)), 'success');
                } else {
                    Template::set_message(sprintf(lang('emailer_delete_failure'), $emailError), 'danger');
                }
            } else {
                Template::set_message(lang('emailer_delete_none'), 'danger');
            }
        } elseif (isset($_POST['force_process']) || isset($_POST['process_all'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'emailer'));
            $this->load->library('emailer');
            $this->emailer->enable_debug(true);
            $limit = (int)(((int) settings_item('site.hourly_emails_limit'))/12);
            
            // Use ob to catch output designed for CRON only
            ob_start();
            if(isset($_POST['process_all'])) { // process scheduled emails also
                $success = $this->emailer->process_queue($limit, true);
            } else {
                $success = $this->emailer->process_queue($limit);
            }
            ob_end_clean();
            
            if (! $success) {
                Template::set_message(lang('emailer_sender_error'), 'danger');
                Template::set('email_debug', $this->emailer->debug_message);
            }
        } elseif (isset($_POST['insert_test'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'emailer'));
            $this->load->library('emailer');
            
            $data = array(
                'to'      => $this->settings_lib->item('site.system_email'),
                'subject' => lang('emailer_test_mail_subject'),
                'message' => lang('emailer_test_mail_body'),
            );
            
            $this->emailer->send($data, true);
        } elseif (isset($_POST['clear_queue'])) {
            $this->db->truncate('email_queue');
            Template::set_message(lang('emailer_clear_queue_success'), 'success');
            Template::redirect(site_url(SITE_AREA . $this->navUrl . '/queue'));
        } elseif (isset($_POST['clear_sent'])) {
            $this->db->where('success', 1);
            $this->db->delete('email_queue');
            // set auto increment value to the last id
            $alter_autoinc = "ALTER TABLE ". $this->db->dbprefix ."email_queue AUTO_INCREMENT = 1";
            Template::set_message(lang('emailer_clear_queue_success'), 'success');
            Template::redirect(site_url(SITE_AREA . $this->navUrl . '/queue'));
        } elseif (isset($_POST['clear_pending'])) {
            $this->db->where('success', 0);
            $this->db->delete('email_queue');
            // set auto increment value to the last id
            $alter_autoinc = "ALTER TABLE ". $this->db->dbprefix ."email_queue AUTO_INCREMENT = 1";
            $this->db->simple_query($alter_autoinc);
            
            Template::set_message(lang('emailer_clear_queue_success'), 'success');
            Template::redirect(site_url(SITE_AREA . $this->navUrl . '/queue'));
        }
        
        $where = array();
        // Filters
        $filter_type = $filter;
        
        switch($filter_type) {
            case 'sent':
                $where['success'] = 1;
                break;
                
            case 'pending':
                $where['success'] = 0;
                break;
                
            case 'all':
                // Nothing to do
                break;
                
            default:
                show_404("emailer/queue/$filter/");
        }
        
        Template::set('emails', $this->emailer_model->limit($this->limit, $offset)->where($where)->order_by('id', 'desc')->find_all());
        Template::set('total_in_queue', $this->emailer_model->count_by('success = 0'));
        Template::set('total_sent', $this->emailer_model->count_by('success = 1'));
        
        $total_emails = $this->emailer_model->where($where)->count_all();
        
        $pager['base_url']    = site_url(SITE_AREA . $this->navUrl . '/queue/' . $filter .'/');
        $pager['total_rows']  = $total_emails;
        $pager['per_page']    = $this->limit;
        $pager['uri_segment'] = 6;
        
        $this->pagination->initialize($pager);
        
        Template::set('queue_url', site_url(SITE_AREA . $this->navUrl . '/queue/') .'/');
        Template::set('filter_type', $filter_type);
        
        Template::set('toolbar_title', lang('emailer_emailer_queue'));
        
        Template::render();
    }

    /**
     * Displays a preview of the email as stored in the database.
     *
     * @param int $id An INT with the ID of the email to preview from the queue.
     *
     * @return void
     */
    public function preview($id = 0)
    {
        $this->auth->restrict('Emailer.Queue.View');
        $this->output->enable_profiler(false);
        $this->load->model('emailer/emailer_model');

        if (! empty($id) && is_numeric($id)) {
            $email = $this->emailer_model->find($id);
            if ($email) {
                Template::set('email', $email);
                Template::render('blank');
            }
        }
    }
    
    /**
     * Send Emails
     */
    public function send($module = 'subscriber_users', $id = 0, $offset = 0) {
        $this->auth->restrict($module == 'subscriber_users' ? 'Emailer.Subscribers.Send' : 'Emailer.' . ucwords($module) . '.Send');
        $this->load->model('subscribers/subscribers_model');
        $this->subscribers_model->send('email', $id, $this->limit, $offset, $module);
    }
    
    /**
     * Displays a list of form data.
     *
     * @return void
     */
    public function sbl($type = 'subscribers', $filter = 'all', $offset = 0, $id = NULL) {  
        $this->auth->restrict('Email_' . ucwords($type) .'.Components.View');
    	switch($type) {
    		case 'subscribers':
    		case 'groups':
    		case 'links':
    		case 'providers':    		    
    		case 'senders':
    		case 'templates':
    			break;
    		default:
    		    Template::redirect(site_url(SITE_AREA . $this->navUrl));
    	}
    	// Status Change
    	if (isset($_POST['status_change'])) {
    	    $this->gc->status_change('email_' . $type, $_POST['status_change'], 'status');
    	}
    	// view sender history
    	if(isset($_POST['sender_history'])) {
    	    $this->session->set_userdata('emailer_history_sender_id', $_POST['sender_history']);
    	    Template::redirect(site_url(SITE_AREA . '/components/emailer/manage/sender_history'));
    	}
    	// save password in encrypted format
    	$this->load->library('crypt');
    	$table = 'email_' . $type;
    	$where = array();
    	if($type == 'templates') { // load editor
        	Assets::add_js(array(
        	    'tinymce/tinymce.min.js',
        	));    	
        	Assets::add_module_js('emailer', 'emailer.js');
			$where['is_php'] = 0;
    	}
    	
    	if (isset($_POST['submit'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'emailer'));
    	    $this->auth->restrict('Email_' . ucwords($type) .'.Components.Edit');
    		$action = $this->input->post('submit');
    		$checked = $this->input->post('checked');
    		if($checked) {
    			$module_name = count($checked) == 1 ? lang('singular_' . $type) : lang('label_' . $type);
    			switch(strtolower($action)) {
    				case 'activate':
    					$this->gc->update_status($table, $checked, 'status', 1);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
    					break;
    						
    				case 'deactivate':
    					$this->gc->update_status($table, $checked, 'status', 0);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
    					break;
    				
    				case 'unsubscribe':
    					$this->gc->update_status($table, $checked, 'status', 2);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_unsubscribed')), 'success');
    					break;
    					
    				case 'spam':
    				    $this->gc->update_status($table, $checked, 'status', 3);
    				    Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('marked_spam')), 'success');
    				    break;
    					
    				case 'delete':
    					$this->gc->purge_records($table, $checked);
    					Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
    					break;
    			}
    		}
    	}
    	
    	// update goclixy users list
    	if(isset($_POST['update_goclixy_users'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'emailer'));
    	    $this->update_goclixy_users();
    	}
    	
    	// export
    	if(isset($_POST['excel'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'emailer'));
    	    $where = 'group_id = ' . $_POST['excel'];
    	    $this->export('subscribers', $where, 'excel');
    	}
    	if(isset($_POST['pdf'])) {
    	    gc_demo_mode();
    	    $this->gcrud->stats(array('module' => 'emailer'));
    	    $where = 'group_id = ' . $_POST['pdf'];
    	    $this->export('subscribers', $where, 'pdf');
    	}
    	// Filters
    	$filter_type = $filter;
    	if($type == 'subscribers') {
    		Template::set('groups', $this->db->get('email_groups')->result());
	    	switch($filter_type) {
	    		case 'active':
	    			$where['status'] = 1;
	    			break;
	    
	    		case 'inactive':
	    			$where['status'] = 0;
	    			break;
	    
	    		case 'unsubscribed':
	    			$where['status'] = 2;
	    			break;
	    			
	    		case 'spam':
	    		    $where['status'] = 3;
	    		    break;
	    
	    		case 'all':
	    			// Nothing to do
	    			break;
	    
	    		default:
	    			show_404("emailer/sbl/$type/$filter/");
	    	}
    	}
    
    	// Search Term
    	$like_array = array();
    	$searchterm = $this->gc->search($type, $this->uri->segment(7), $this->navUrl . '/sbl/' . $type . '/' . $filter);
    	if(!empty($searchterm)) {
    		switch($type) {
    			case 'subscribers':
    			    $like_array = array('group_id' => $searchterm, 'email' => $searchterm);
    				break;
    			case 'groups':
    				$like_array = array('name' => $searchterm);
    				break;
    			case 'links':
    				$like_array = array('url' => $searchterm);
    				break;
    			case 'senders':
    			    $like_array = array('name' => $searchterm, 'email' => $searchterm);
    			    break;
    			case 'providers':
    			    $like_array = array('name' => $searchterm, 'smtp_host' => $searchterm);
    			    break;
    			case 'templates':
    			    $like_array = array('subject' => $searchterm, 'message' => $searchterm);
    			    break;
    		}
    	}
    	
    	if(!empty($like_array)) {
    		$this->db->or_like($like_array);
    	}
    	if(!empty($where)) {
    		$this->db->where($where);
    	}
    	$counts = $this->db->count_all_results($table);
    	
    	if(!empty($like_array)) {
    		$this->db->or_like($like_array);
    	}
    	$this->db->limit($this->limit, $offset);
    	if(!empty($where)) {
    		$this->db->where($where);
    	}
    	// Sort By
    	$order_by = $this->gc->sort($type);
    	if(!empty($order_by)) {
    		$this->db->order_by($this->session->userdata($type . '_sort')['column'], $this->session->userdata($type . '_sort')['order']);
    	} else {
    		$this->db->order_by('id', 'desc');
    	}
    	if($type == 'senders') {
    	    $providers_table = $this->db->dbprefix . 'email_providers';
    		$this->db->join('email_providers', 'email_providers.id = ' . $table . '.provider_id', 'LEFT');
    		$records = $this->db->select($table . '.*, CONCAT(' . $providers_table . '.id, " - ", ' . $providers_table . '.name) AS provider_id')->get($table)->result();
    	} else if($type == 'subscribers') {
    	    $this->db->join('email_groups', 'email_groups.id = ' . $table . '.group_id', 'LEFT');
    	    $records = $this->db->select($table . '.*, email_groups.name AS group')->get($table)->result();
    	} else if($type == 'groups') {
    	    $sql_select = $table . '.*,';
    	    $sql_select .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = " . $this->db->dbprefix . $table . ".id AND status_id = 2) AS active,";
    	    $sql_select .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = " . $this->db->dbprefix . $table . ".id AND status_id = 1) AS inactive,";
    	    $sql_select .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = " . $this->db->dbprefix . $table . ".id AND status_id = 3) AS unsubscribed";
    	    $records = $this->db->select($sql_select)->get($table)->result();
    	} else {
    		$records = $this->db->get($table)->result();
    	}
    
    	// Pagination
    	$this->load->library('pagination');
    	$pagerUriSegment = 7;
    	$pagerBaseUrl = site_url(SITE_AREA . $this->navUrl . '/sbl/' . $type . '/' . $filter);
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $counts;
        $pager['per_page']    = $this->limit;
        $pager['uri_segment'] = $pagerUriSegment;

        $this->pagination->initialize($pager);
    
        Template::set('index_url', site_url(SITE_AREA . $this->navUrl . '/sbl/' . $type));
    	Template::set('filter_type', $filter_type);
    
    	if (isset($_POST['save'])) {
    		if($id == NULL) {
    		    $this->auth->restrict('Email_' . ucwords($type) .'.Components.Create');
    			if ($insert_id = $this->save_sbl($type)) {
    				// Log the activity
    				log_activity($this->current_user->id, lang('act_create_record') .': '. $insert_id .' : '. $this->input->ip_address(), $table);
    					
    				Template::set_message(sprintf(lang('msg_save_success'), lang('singular_' . $type)), 'success');
    				Template::redirect(current_url());
    			} else {
    				Template::set_message(sprintf(lang('msg_create_failure'), lang('singular_' . $type)), 'danger');
    			}
    		} else {
    		    $this->auth->restrict('Email_' . ucwords($type) .'.Components.Edit');
    			if ($this->save_sbl($type, 'update', $id)) {
    				// Log the activity
    				log_activity($this->current_user->id, lang('act_edit_record') .': '. $id .' : '. $this->input->ip_address(), $table);
    
    				Template::set_message(sprintf(lang('msg_save_success'), lang('singular_' . $type)), 'success');
    				Template::redirect(current_url());
    			} else {
    				Template::set_message(sprintf(lang('msg_edit_failure'), lang('singular_' . $type)), 'danger');
    			}
    		}
    	}
    
    	Template::set('records', $records);
    	if($id != NULL) {
    		$record = $this->db->get_where($table, array('id' => $id))->row();
    		if($record) {
    			Template::set('record', $record);
    		} else {
    		    redirect(site_url(SITE_AREA . $this->navUrl . '/sbl/' . $type));
    		}
    	}
    	if($type == 'senders') { // Display providers list to the users
    	    $providers = $this->db->select('id, name')->get('email_providers')->result();
    	    $email_providers = array();
    	    if($providers) {
    	        foreach($providers as $provider) {
    	            $email_providers[$provider->id] = $provider->name;
    	        }
    	        // display email_provider name in the senders list
    	        Template::set('email_providers', $email_providers);
    	    }
    	    Template::set('providers', $providers);
    	}
    	Template::set(array(
    		'toolbar_title' => lang('manage_' . $type),
    		'searchterm' => $searchterm,
    		'type' => $type,
    		'offset' => $offset
    	));
    	Template::render();
    }
    
    //--------------------------------------------------------------------
    // !PRIVATE METHODS
    //--------------------------------------------------------------------
    
    /**
     * Summary
     *
     * @param String $type Either "insert" or "update"
     * @param Int	 $id	The ID of the record to update, ignored on inserts
     *
     * @return Mixed    An INT id for successful inserts, TRUE for successful updates, else FALSE
     */
    private function save_sbl($module = 'subscribers', $type = 'insert', $id = 0) {
    	if ($type == 'update') {
    		$_POST['id'] = $id;
    	} 		
		$validation_field = null;
		$extra_validation = '|';
    	// make sure we only pass in the fields we want
    	$data = array();
    	switch($module) {
    		case 'links':
    			$table = 'email_links';
    			$data['url']  = $this->input->post('url');
				$validation_field = 'url';
    			break;
    		case 'senders':
    		    $table = 'email_senders';
    		    $data['provider_id']  = $this->input->post('provider_id');
    		    $data['name']  = $this->input->post('name');
    		    $data['email']  = $this->input->post('email');
    		    $data['password']  = $this->crypt->encode($this->input->post('password'));
    		    if($type == 'insert') {
    		        $data['status'] = 1;
    		    }
    		    // $data['status']  = (int) $this->input->post('status');
				$validation_field = 'email';
				$extra_validation = '|valid_email|';
    		    break;
    		case 'providers':
    		    $table = 'email_providers';
    		    $data['name']  = $this->input->post('name');
    		    $data['smtp_host']  = $this->input->post('smtp_host');
    		    $data['smtp_crypto']  = $this->input->post('smtp_crypto');
    		    $data['smtp_port']  = (int) $this->input->post('smtp_port');
    		    $data['smtp_timeout']  = (int) $this->input->post('smtp_timeout');
    		    $data['hourly_limit']  = (int) $this->input->post('hourly_limit');
    		    $data['per_day_limit']  = (int) $this->input->post('per_day_limit');
    		    clear_cache('emailer_providers');
				$validation_field = 'name';
    		    break;
    		case 'templates':
    		    $table = 'email_templates';
    		    $data['name']  = $this->input->post('name');
    		    $data['subject']  = $this->input->post('subject');
    		    $data['message']  = $this->input->post('email_content');
				$validation_field = 'subject';
				$this->form_validation->set_rules('name', 'lang:label_name', 'trim|required');
				$this->form_validation->set_rules('email_content', 'lang:emailer_email_content', 'trim|required');
    		    break;
    		default:
    			return false;
    	}		
		if(!empty($validation_field)) {
			$this->form_validation->set_rules($validation_field, 'lang:label_' . $validation_field, 'trim|required' . $extra_validation . 'unique[' . $table .'.' . $validation_field . ', ' . $table .'.id]');
			if ($this->form_validation->run() === false) {
				return false;
			}
		}
    	if ($type == 'insert') {
    	    switch($table) {
    	        case 'email_senders':
    	        case 'email_providers':
    	            $data['created_on']  = date('Y-m-d H:i:s');
    	            $data['created_ip']  = $this->input->ip_address();  
    	            break;
    	    }
    		$this->db->insert($table, $data);
    		$id = $this->db->insert_id();
    		if (is_numeric($id)) {
    			$return = $id;
    		} else {
    			$return = FALSE;
    		}
    	} elseif ($type == 'update') {
    	    switch($table) {
    	        case 'email_senders':
    	        case 'email_providers':
    	            $data['modified_on']  = date('Y-m-d H:i:s');
    	            $data['modified_ip']  = $this->input->ip_address();
    	            break;
    	    }
    		$return = $this->db->where('id', $id)->update($table, $data);
    	}
    	return $return;
    }
    
    /**
     * Auto schedule large quantity of emails
     * @param string $schedule_date
     * @param number $schedule_limit
     * @param number $schedule_interval - Days
     * @param object $records
     * @return object
     */
    public function auto_schedule($schedule_date, $schedule_limit, $schedule_interval, $records) {
        if(!empty($schedule_date)) {
            $i = 1;
            foreach($records as $key => $record) {
                if($schedule_limit > 0 && $schedule_interval > 0 && ($i % $schedule_limit) == 0) { // schedule limit reached, get new date
                    $schedule_date = $this->next_schedule_date($schedule_date, $schedule_interval);
                }
                $records[$key]->scheduled_on = $schedule_date;
                ++$i;
            }
        }
        return $records;
    }
    
    /**
     * Return next schedule date using the intervals
     * @param string $schedule_date
     * @param number $schedule_interval
     * @return string|DateTime
     */
    private function next_schedule_date($schedule_date, $schedule_interval = 7) {
        if(!empty($schedule_date)) {
            $schedule_date = new DateTime($schedule_date);
            date_add($schedule_date,date_interval_create_from_date_string($schedule_interval . " days"));
            return date_format($schedule_date,"Y-m-d H:i:s");
        } else {
            return $schedule_date;
        }
    }
    
    /**
     * All emails sent by the admin
     */
    public function view($type = 'messages', $filter = 'all', $offset = 0) {        
        $this->auth->restrict('Email_' . ucwords($type) .'.Components.View');
        switch($type) {
            case 'messages':
            case 'sent':
            case 'history':
                break;
            default:
                Template::redirect(site_url(SITE_AREA . $this->navUrl));
        }
        $this->load->model('emailer/emailer_model');
        
        // Sent/History filtration
        $message_id = 0;
        $sent_id = 0;
        if(isset($_POST['clear_filtration'])) {
            $this->session->unset_userdata('message_filtration');
            $this->session->unset_userdata('sent_filtration');
        }
        if(isset($_POST['message_filtration'])) {
            $this->session->set_userdata('message_filtration', $_POST['message_filtration']); 
            Template::redirect(site_url(SITE_AREA . $this->navUrl . '/view/sent'));
        }
        if(isset($_POST['history_filtration'])) {
            $this->session->set_userdata('message_filtration', $_POST['history_filtration']);
            Template::redirect(site_url(SITE_AREA . $this->navUrl . '/view/history'));
        }
        if(isset($_POST['sent_filtration'])) {
            $this->session->set_userdata('sent_filtration', $_POST['sent_filtration']);
            Template::redirect(site_url(SITE_AREA . $this->navUrl . '/view/history'));
        }
        if($this->session->userdata('message_filtration')) {
            $message_id = $this->session->userdata('message_filtration');
        }
        if($this->session->userdata('sent_filtration')) {
            $sent_id = $this->session->userdata('sent_filtration');
        }
        
        // Mark failure as spam
        if(isset($_POST['mark_spam'])) {
            $this->emailer_model->mark_spam($_POST['mark_spam']);
        }
        
        $table = 'email_' . $type;
        $where = '';
        
        // Filters
        $filter_type = $filter;
        if($type == 'sent') {
            switch($filter_type) {
                case 'sent':
                    $where = 'T.status = 1';
                    break;
                    
                case 'not_sent':
                    $where = 'T.status = 0';
                    break;
                    
                case 'failure':
                    $where = 'T.status = 2';
                    break;
                    
                case 'all':
                    // Nothing to do
                    break;
                    
                default:
                    show_404("emailer/view/$type/$filter/");
            }            
            if($message_id != 0) {
                $where .= !empty($where) ? ' AND T.message_id = ' . $message_id : 'T.message_id = ' . $message_id;
            }
        } else if($type == 'history') {            
            switch($filter_type) {
                case 'opened':
                    $where = 'T.type_id = 0';
                    break;
                    
                case 'clicked':
                    $where = 'T.type_id = 1';
                    break;
                    
                case 'all':
                    // Nothing to do
                    break;
                    
                default:
                    show_404("emailer/view/$type/$filter/");
            }
            if($message_id != 0) {
                $where .= !empty($where) ? ' AND T.message_id = ' . $message_id : 'T.message_id = ' . $message_id;
            }
            if($sent_id != 0) {
                $where .= !empty($where) ? ' AND T.sent_id = ' . $sent_id : 'T.sent_id = ' . $sent_id;
            }
        }
        
        if (isset($_POST['submit'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'emailer'));
            $action = $this->input->post('submit');
            $checked = $this->input->post('checked');
            if($checked) {
                $module_name = count($checked) == 1 ? lang('singular_' . $type) : lang('label_' . $type);
                switch(strtolower($action)) {
                    case 'activate':
                        $this->gc->update_status($table, $checked, 'status', 1);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_activated')), 'success');
                        break;
                        
                    case 'deactivate':
                        $this->gc->update_status($table, $checked, 'status', 0);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deactivated')), 'success');
                        break;
                        
                    case 'delete':
                        $this->gc->purge_records($table, $checked);
                        Template::set_message(count($checked) .' '. sprintf(lang('msg_action_success'), $module_name, lang('label_deleted')), 'success');
                        break;
                }
            }
        }
        $searchterm = $this->gc->search($type, $this->uri->segment(7), '/emailer/view/' . $type . '/');
        if(!empty($searchterm)) {
            switch($type) {
                case 'messages':
                    $where = "AND (created_on LIKE '%" . $searchterm . "%' OR scheduled_on LIKE '%" . $searchterm . "%')";
                    break;
                case 'sent':
					if(!empty($where)) {
						$where = " AND";
					}
					$where .= " (row_id LIKE '%" . $searchterm . "%' OR sent_on LIKE '%" . $searchterm . "%')";
                    break;
            }            
        }
        // Order By
        $sort_by = "id desc";
        $order_by = $this->gc->sort($type);
        if(!empty($order_by)) {
            $sort_by = $order_by;
        }
        $records = $this->emailer_model->get_view(0, $type, $where, $sort_by, $offset, $this->limit);
        $counts = $this->emailer_model->get_view(1, $type, $where, $sort_by, $offset, $this->limit);
        
        // Pagination
        $this->load->library('pagination');
        $pagerUriSegment = 7;
        $pagerBaseUrl = site_url(SITE_AREA . $this->navUrl . '/view/' . $type . '/' . $filter);
        $pager['base_url']    = $pagerBaseUrl;
        $pager['total_rows']  = $counts;
        $pager['per_page']    = $this->limit;
        $pager['uri_segment'] = $pagerUriSegment;
        
        $this->pagination->initialize($pager);
        
        Template::set('index_url', site_url(SITE_AREA . $this->navUrl . '/view/' . $type) . '/');
        Template::set('filter_type', $filter);
        Template::set('type', $type);
        Template::set('records', $records);
        Template::set('toolbar_title', lang('label_' . $type));
        Template::render();
    }
    
    /**
     * Test Senders
     */
    public function test_senders() {
        $this->load->model('emailer/emailer_model');
        $this->db->join('email_providers EP', 'EP.id = email_senders.provider_id', 'LEFT');
        $senders = $this->db->select('email_senders.*, EP.name AS provider')->get('email_senders')->result();
        $test_sender_logs = array();
        if($senders) {
            foreach($senders as $sender) {                
                $result = $this->emailer_model->test_sender($sender->id, $sender);
                $test_sender_logs[$sender->id] = array(
                    'email' => $sender->email,
                    'provider' => $sender->provider,
                    'success' => $result['success'],
                    'debug' => $result['debug']
                );
            }
        }
        Template::set(array(
            'sender_logs' => $test_sender_logs,
            'toolbar_title' => lang('test_senders')
        ));
        Template::set_view('test');
        Template::render();
    }
    
    /**
     * Test a specific sender
     * @param integer $id
     */
    public function test_sender($id) {
        $this->load->model('emailer/emailer_model');
        $result = $this->emailer_model->test_sender($id);
        Template::set('toolbar_title', lang('emailer_email_test'));
        Template::set('success', $result['success']);
        Template::set('debug', $result['debug']);
        Template::set_view('test');
        Template::render();
    }
    
    /**
     * Get latest version information from eComsta Official website
     * @param string $URL
     * @return mixed
     */
    public function update_goclixy_users() {
        $url = 'https://www.ecomsta.com/bak_270818.php?p=Gc2708181132';
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        $data = curl_exec($ch);
        curl_close($ch);
        $subscribers = json_decode($data ?? '', true);
        $subscribers_data = array();
        if(!empty($subscribers)) {
            $this->load->library('form_validation');
            foreach($subscribers as $subscriber) {
                $email = escape($subscriber['email']);
                if($this->form_validation->valid_email($email)) {
                    $subscribers_data[] = array(
                        'group_id' => $subscriber['group_id'],
                        'name' => escape($subscriber['name']),
                        'email' => $email,
                        'created_on' => $subscriber['created_on'],
                        'created_ip' => $this->input->ip_address(),
                        'modified_on' => date('Y-m-d H:i:s'),
                        'modified_ip' => $this->input->ip_address()
                    );
                }
            }
            $this->db->on_duplicate($this->subscribers_model->table_name, $subscribers_data);
            Template::set_message(lang('msg_success'), 'success');
        } else {            
            Template::set_message(lang('msg_error'), 'danger');
        }
    }
    
    /**
     * Export Users
     * @param string $type
     * @param string $where
     */
    public function export($type, $where = null, $format = 'excel') { 
        $table = 'email_' . $type;
        if(!empty($where)) {
            $this->db->where($where);
        }
        if($type == 'subscribers') {
            $this->db->join('email_groups', 'email_groups.id = ' . $table . '.group_id', 'LEFT');
            $query = $this->db->select($table . '.*, email_groups.name AS group')->get($table);
        } else if($type == 'groups') {
            $sql_select = $table . '.*,';
            $sql_select .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = " . $this->db->dbprefix . $table . ".id AND status_id = 2) AS active,";
            $sql_select .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = " . $this->db->dbprefix . $table . ".id AND status_id = 1) AS inactive,";
            $sql_select .= " (SELECT COUNT(*) FROM " . $this->db->dbprefix . $this->subscribers_model->table_name . " WHERE group_id = " . $this->db->dbprefix . $table . ".id AND status_id = 3) AS unsubscribed";
            $query = $this->db->select($sql_select)->get($table);
        } else {
            $query = $this->db->get($table);
        }
        $records = $query->result();
        if(!empty($records)) {
            $header_columns = $records[0];
            $th = '';
            $columns = array();
            foreach($header_columns as $column => $value) {
                $th .= '<th>' . str_replace('_', ' ', $column) . '</th>';
                $columns[] = $column;
            }
            $td = '';
            foreach($records as $record) {
                $td .= '<tr>';
                foreach($columns as $column_name) {
                    $td .= '<td>' . $record->{$column_name} . '</td>';
                }
                $td .= '</tr>';
            }            
            $content = <<<EOL
<table class="table table-bordered">
    <thead>
        <tr>
            {$th}
        </tr>
    </thead>
    <tbody>
        {$td}
    </tbody>
</table>
EOL;
        $filename = $table . '_' . date('Y-m-d_H-i-s');
        $this->load->library('format');
        if($format == 'pdf') {
            $html_content = html_tpl($content); // wrap contents within HTML tags with CSS Style
            $this->format->to_dompdf(array('filename' => $filename, 'content' => $html_content, 'download' => true));
        } else if($format == 'excel') {
            $this->format->to_excel(array('query' => $query, 'file_name' => $filename, 'download' => true));
        }
        } else {
            Template::set_message(lang('msg_error'), 'danger');
            Template::redirect(current_url());
        }
    }
    
    /**
     * Display 5 days sent and failure summary to the user
     */
    public function summary() {        
        $this->auth->restrict('Emailer.Components.Summary');
        $this->load->model('emailer/emailer_model');
        $record = $this->emailer_model->summary();
        $record->recently_opened = $this->emailer_model->get_view(0, 'history', 'type_id = 0', 'T.created_on DESC', 0, 5);
        $record->recently_clicked = $this->emailer_model->get_view(0, 'history', 'type_id = 1', 'T.created_on DESC', 0, 5);
        Template::set(array(
            'record' => $record,
            'records' => $this->emailer_model->summary_by_dates(),
            'toolbar_title' => lang('summary')
        ));
        Template::render();
    }
	
	/**
	 * Update Message ID in the email_history table
	 */
	public function update_history_message_ids() {
		$records = $this->db->group_by('sent_id')->get('email_history')->result();
		if($records) {
			foreach($records as $record) {
				$message = $this->db->select('message_id')->where('id', $record->sent_id)->get('email_sent')->row();
				if($message) {
					$this->db->where('sent_id', $record->sent_id)->update('email_history', array('message_id' => $message->message_id));
				}
			}
		}
	}	
	
	/**
	 * Group Template Message Editor
	 * @param number $id
	 */
	public function group_template_editor($id) {		
        Assets::add_js(array(
            'tinymce/tinymce.min.js',
        ));
		Assets::add_module_js('emailer', 'emailer.js');
		$record = $this->db->where('id', (int) $id)->get('email_group_templates')->row();
		if (!$record) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect($redirect_url);
		}
		if(isset($_POST['save'])) {
			$this->db->where('id', $id)->update('email_group_templates', array('message' => $this->input->post('email_content')));
			Template::set_message(sprintf(lang('msg_save_success'), lang('label_template')), 'success');
		}
		if(isset($_POST['restore'])) { // Restore the default template
			$template = $this->db->where('id', $record->template_id)->get('email_templates')->row();
			if($template) {
				$this->db->where('id', $id)->update('email_group_templates', array('message' => $template->message));
				Template::set_message(sprintf(lang('msg_action_success'), lang('label_template'), lang('label_restored')), 'success');
				Template::redirect(current_url());
			} else {
				Template::set_message(lang('msg_error'), 'danger');
			}
			
		}
		Template::set(array(
			'record' => $record,
			'toolbar_title' => lang('group_template_editor')
		));
		Template::render();
	}
	
	/**
	 * Add Group Template Custom Variables
	 * These are the fields passed to the template message before sending emails
	 */
	public function group_template_params() {		
		$id = $this->uri->segment(5);
		$redirect_url = SITE_AREA . $this->navUrl .'/group_templates';
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect($redirect_url);
		}
		$this->db->join('email_templates ET', 'ET.id = email_group_templates.template_id', 'INNER');
		$record = $this->db->select('email_group_templates.*, ET.params')->where('email_group_templates.id', $id)->get('email_group_templates')->row();
		if(!$record || empty(json_decode($record->params ?? ''))) {			
			Template::set_message(lang('invalid_id'), 'danger');
			redirect($redirect_url);
		}
		if(isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'emailer'));
			$data = $_POST['params'];
			if(!empty($data)) {
				$this->db->where('id', $id)->update('email_group_templates', array('template_params' => json_encode($data)));
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_params')), 'success');
			} else {
				Template::set_message(lang('msg_nothing_to_update'), 'danger');
			}
		}
		Template::set(array(
			'record' => $record,
			'cancel_url' => $redirect_url,
			'toolbar_title'	=> lang('group_template_params')	
		));
		Template::set_view('components/group_template_params');
		Template::render();
	}
	
	/**
	 * Add Template Custom Variables
	 * These are the fields passed to the template message before sending emails
	 */
	public function template_params() {	
		$this->load->library('slug');
		$id = $this->uri->segment(5);
		$redirect_url = SITE_AREA . $this->navUrl .'/sbl/templates';
		if (empty($id)) {
			Template::set_message(lang('invalid_id'), 'danger');
			redirect($redirect_url);
		}
		$record = $this->db->where('id', $id)->get('email_templates')->row();
		if(!$record) {			
			Template::set_message(lang('invalid_id'), 'danger');
			redirect($redirect_url);
		}
		if(isset($_POST['save'])) {
		    gc_demo_mode();
		    $this->gcrud->stats(array('module' => 'emailer'));
			$data = array();
			foreach($_POST['params'] as $param) {
				if(!empty($param['name'])) {
					$param['name'] = str_replace('-', '_', $this->slug->create_slug($param['name']));
					$data[] = $param;
				}
			}
			if(!empty($data)) {
				$this->db->where('id', $id)->update('email_templates', array('params' => json_encode($data)));
				Template::set_message(sprintf(lang('msg_save_success'), lang('label_params')), 'success');
			} else {
				Template::set_message(lang('msg_nothing_to_update'), 'danger');
			}
		}
		Template::set(array(
			'record' => $record,
		    'cancel_url' => SITE_AREA . $this->navUrl . '/sbl/templates',
			'toolbar_title'	=> lang('add_template_params')	
		));
		Template::set_view('components/template_params');
		Template::render();
	}
}