<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Bonfire
 *
 * An open source project to allow developers to jumpstart their development of
 * CodeIgniter applications.
 *
 * @package   Bonfire
 * @author    Bonfire Dev Team
 * @copyright Copyright (c) 2011 - 2015, Bonfire Dev Team
 * @license   http://opensource.org/licenses/MIT The MIT License.
 * @link      http://cibonfire.com
 * @since     Version 1.0
 * @filesource
 */

/**
 * Permissions Settings Context
 *
 * Allows the management of the Bonfire permissions.
 *
 * @package Bonfire\Modules\Permissions\Controllers\Settings
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs
 *
 */
class Settings extends Admin_Controller
{
    /**
     * Sets up the require permissions and loads required classes
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();

        $this->auth->restrict('Bonfire.Permissions.View');
        $this->auth->restrict('Bonfire.Permissions.Manage');

        $this->load->library('form_validation');
        $this->load->helper('form');
        $this->load->helper('url');
        $this->load->model('permission_model');
        $this->lang->load('permissions');
        $this->load->helper('inflector');

        Template::set_block('sub_nav', 'settings/_sub_nav');

    }

    /**
     * Displays a list of all permissions with pagination
     *
     * @return void
     */
    public function index()
    {
        // Deleting anything?
        if (isset($_POST['delete'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'permissions'));
            $checked = $this->input->post('checked');

            if (! empty($checked) && is_array($checked)) {
                $result = false;
                foreach ($checked as $pid) {
                    $result = $this->permission_model->delete($pid);
                }

                if ($result) {
                    Template::set_message(count($checked) .' '. lang('permissions_deleted') .'.', 'success');
                } else {
                    Template::set_message(lang('permissions_del_failure') . $this->permission_model->error, 'danger');
                }
            } else {
                Template::set_message(lang('permissions_del_error') . $this->permission_model->error, 'danger');
            }
        }

        $offset = $this->input->get('per_page');
        $limit = $this->settings_lib->item('site.list_limit');
        
        // Search
        $like_array = array();
        $searchterm = $this->gc->search('permissions', $this->uri->segment(6), '/settings/permissions/');
        if(!empty($searchterm)) {
        	$like_array = array(
        		'name' => $searchterm
        	);
        }
        
        // Order By
        $order_by = $this->gc->sort('permissions');
        if(!empty($order_by)) {
        	$this->permission_model->order_by($this->session->userdata('permissions_sort')['column'], $this->session->userdata('permissions_sort')['order']);
        } else {
        	$this->permission_model->order_by('permission_id', 'desc');
        }
        
        if(!empty($like_array)) {
        	$this->db->or_like($like_array);
        }
        $permissions = $this->permission_model->limit($limit, $offset)->find_all();
        
        if(!empty($like_array)) {
        	$this->db->or_like($like_array);
        }
        $total = $this->permission_model->count_all();

        // Pagination
        $this->load->library('pagination');

        $pager['base_url']            = current_url() .'?';
        $pager['total_rows']          = $total;
        $pager['per_page']            = $limit;
        $pager['page_query_string']   = true;

        $this->pagination->initialize($pager);

        Template::set('results', $permissions);
        Template::set("toolbar_title", lang("permissions_manage"));
        Template::render();
    }

    /**
     * Create a new permission in the database
     *
     * @return void
     */
    public function create()
    {
        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'permissions'));
            if ($this->savePermissions()) {
                Template::set_message(lang("permissions_create_success"), 'success');
                redirect(SITE_AREA . '/settings/permissions');
            }
        }

        Template::set('toolbar_title', lang("permissions_create_new_button"));
        Template::set_view('settings/permission_form');
        Template::render();
    }

    /**
     * Edit a permission record
     *
     * @return void
     */
    public function edit()
    {
        $id = (int) $this->uri->segment(5);
        if (empty($id)) {
            Template::set_message(lang("permissions_invalid_id"), 'danger');
            redirect(SITE_AREA . '/settings/permissions');
        }

        if (isset($_POST['save'])) {
            gc_demo_mode();
            $this->gcrud->stats(array('module' => 'permissions'));
            if ($this->savePermissions('update', $id)) {
                Template::set_message(lang("permissions_edit_success"), 'success');
            }
        }

        Template::set('permissions', $this->permission_model->find($id));
        Template::set('toolbar_title', lang("permissions_edit_heading"));
        Template::set_view('settings/permission_form');
        Template::render();
    }

    /**
     * Save the permission record to the database
     *
     * @param string $type The type of save operation (insert or edit)
     * @param int    $id   The record ID in the case of edit
     *
     * @return bool
     */
    private function savePermissions($type = 'insert', $id = 0) {
        gc_demo_mode(array('redirect' => 'permissions'));
        $this->form_validation->set_rules('name', 'lang:bf_name', 'required|trim|max_length[50]');
        $this->form_validation->set_rules('description', 'lang:bf_description', 'trim|max_length[100]');
        $this->form_validation->set_rules('status', 'lang:bf_status', 'required|trim');
        if ($this->form_validation->run() === false) {
            return false;
        }

        unset($_POST['submit'], $_POST['save']);

        if ($type == 'insert') {
            $id = $this->permission_model->insert($_POST);
            return is_numeric($id);
        } elseif ($type == 'update') {
            return $this->permission_model->update($id, $_POST);
        }

        // Unsupported value for $type.
        return false;
    }
    
    /**
     * Manage Permissions
     * @param string $module
     * @param string $filter
     * @param number $offset
     */
    /*public function manage() {
        $modules = Modules::list_modules(true);
        $configs = array();
        
        foreach ($modules as $module) {
            $configs[$module] = Modules::config($module);
            
            if (! isset($configs[$module]['name'])) {
                $configs[$module]['name'] = ucwords($module);
            } elseif (strpos($configs[$module]['name'], 'lang:') === 0) {
                // If the name is configured, check to see if it is a lang entry
                // and, if it is, pull it from the application_lang file.
                $configs[$module]['name'] = lang(str_replace('lang:', '', $configs[$module]['name']));
            }
        }
        // Sort the module list (by the name of each module's folder)
        ksort($configs);
        $records = $this->db->get('permission_modules')->result();
        $data = array();
        $saved_modules = array();
        if($records) {
            foreach($records as $record) {
                $saved_modules[] = $record->name;
            }
        }
        $new_modules = array_diff($modules, $saved_modules);
        foreach($new_modules as $new_module) {
            $data[] = array(
                'name' => $new_module,
                'status' => 1,
                'modified_on' => date('Y-m-d H:i:s'),
                'modified_by' => $this->auth->user_id(),
                'modified_ip' => $this->input->ip_address()
            );
        }
        if(!empty($data)) {
            $this->db->insert_batch('permission_modules', $data);
        }
        Template::set(array(
            'configs' => $configs,
            'toolbar_title' => 'Manage'
        ));
        Template::render();
    }
    
    /**
     * Enable/Disable Modules
     * @param int $id
     */
    /*private function change_status($id) {
        $record = $this->db->where('id', $id)->get('permission_modules')->row();
        if($record->status == 1) { // disable module
            
        } else { // enable module
            
        }
    }
    
    public function test($module = 'activities', $action = 'insert') {
        $modules = Modules::files($module);
        $module_controllers = array();
        if(!empty($modules[$module]['controllers'])) {
            $controllers = $modules[$module]['controllers'];
            foreach($controllers as $controller) {
                $module_controller = str_replace('.php', '', $controller);
                if(strtolower($module_controller) != $module) {
                    $module_controllers[] = $module_controller;
                }
            }
        }
        $permission_names = array();
        switch($module) {
            case 'activities':
                $permission_names[] = "name LIKE '%Bonfire.Activities.View%'";
                break;
            default:                
                $module = ucwords(strtolower($module));
                foreach($module_controllers as $controller_module) {
                    $permission_names[] = "name LIKE '" . $module . "." . $controller_module . ".View%'";
                }                
                break;
        }
        // get all permissions from the permissions table using this name
        $where = implode(' OR ', $permission_names);
        $records = $this->db->where($where)->get('permissions')->result();
        if($records) {
            foreach($records as $record) {
                if($action == 'insert') {
                    // check whether permission exists or not
                    $counts = $this->db->where(array('permission_id' => $record->permission_id, 'role_id' => 1))->count_all_results('role_permissions');                    
                    if($counts > 0) {
                        
                    }
                }
            }
        }
    }*/
}
