<?php defined('BASEPATH') || exit('No direct script access allowed');

/**
 * Translate Module Library
 *
 * Provides methods to retrieve translations from external sites.
 *
 * @uses Bonfire\Libraries\Curl (CodeIgniter cURL library from Phil Sturgeon)
 *
 * @package Bonfire\Modules\Translate\Libraries\Translate_lib
 * @author  Bonfire Dev Team
 * @link    http://cibonfire.com/docs
 * @todo    Update Link to a Docs/Guides on the translate_lib methods
 */
use Stichoza\GoogleTranslate\GoogleTranslate;
class Translate_lib {
    /** @var object A pointer to the CodeIgniter instance. */
    protected $ci;
    
    /** @var string Last translation result. */
    public $lastResult = '';
    
    /** @var string Language engine for translation. */
    private $langEngine = 'google';
    
    /** @var string Language code translating from. */
    private $langFrom;
    
    /** @var string Language code translating to. */
    private $langTo;
    
    /** @var array URL formats (Google Translate, others). */
    private static $urlFormat = array(
        'google' => array(
            'url' => 'http://translate.google.com/translate_a/t?client=t&text=%s&hl=en&sl=%s&tl=%s&ie=UTF-8&oe=UTF-8&multires=1&otf=1&pc=1&trs=1&ssel=3&tsel=6&sc=1',
            'langcodes' => array(''),
        ),
    );
    
    /** cache variable */
    protected $gc_translate_cache;
    
    // ------------------------------------------------------------------------
    
    /**
     * The Translate Construct allows optionally passing settings in array and stores them
     *
     * @param array $options containing:
     * 'engine' Language engine for translation (Optional)
     * string 'from' Language code translating from (Optional)
     * string 'to' Language code translating to (Optional)
     *
     * @return void
     */
    public function __construct($options = array()) {
        $this->ci =& get_instance();
        
        // Load Zend Caching
        $this->ci->load->library('slug');
        $this->ci->load->library('gzf_cache', array('lifetime' => 900));
        if(method_exists($this->ci->gzf_cache, 'get_instance')) {
            $this->gc_translate_cache = $this->ci->gzf_cache->get_instance();
        }
        
        if (isset($options['engine'])) {
            $this->setEngine($options['engine']);
        }
        
        if (isset($options['from'])) {
            if (isset($options['to'])) {
                $this->setLang($options['from'], $options['to']);
            } else {
                $this->setLang($options['from']);
            }
        } elseif (isset($options['to'])) {
            $this->setLang($this->langFrom, $options['engine']);
        }
        
    }
    
    /**
     * Set language engine
     *
     * @param string $engine Language Engine
     *
     * @return string
     */
    public function setEngine($engine) {
        $this->langEngine = $engine;
        return $this;
    }
    
    /**
     * Set language codes
     *
     * @param string $from Language code
     * @param string $to Language code
     *
     * @return string
     */
    public function setLang($from, $to = 'en') {
        $this->langFrom = $from;
        $this->langTo = $to;
        return $this;
    }
    
    /**
     * Retrieves a translation.
     *
     * @param string $text Text to be translated
     *
     * @return string|bool  Translated text OR false if failure
     */
    public function translate_old($text) {
        $this->ci->load->library('curl');
        $url = sprintf(self::$urlFormat[$this->langEngine]['url'], rawurlencode($text), $this->langFrom, $this->langTo);
        $result = $this->ci->curl->simple_get($url);
        if (! $result) {
            return false;
        }
        
        // Clean result to prevent JSON syntax errors
        $result = preg_replace('!,+!', ',', $result); // Remove repeated commas
        $result = str_replace('[,', '[', $result); // Remove extra comma after bracket
        $resultArray = json_decode($result, true);
        if (! isset($resultArray[0])) {
            return false;
        }
        
        $finalResult = '';
        foreach ($resultArray[0] as $results) {
            $finalResult .= $results[0];
        }
        
        return $this->lastResult = $finalResult;
    }
    
    /**
     * Text translate using: https://github.com/Stichoza/google-translate-php
     * @param string $text Text to be translated
     *
     * @param string $proxy Google has become stricter, and it seems like they keep lowering the number of allowed requests per IP per a certain amount of time.
     * Try sending less requests to stay under the radar, or change your IP frequently
     *
     * @return string|bool  Translated text OR false if failure
     */
    public function translate_using_session($text, $proxy = null) {
        if (version_compare(PHP_VERSION, '5.5.0') >= 0) {
            $gTranslate = new GoogleTranslate();
            // use session to avoid conversion of the same text again and again
            // session does not exist, set it
            if(!$this->ci->session->userdata('stichoza')) {
                $this->ci->session->set_userdata('stichoza', array($this->langTo => array()));
            }
            // session exists but language is not in the session
            $stichoza = (array) $this->ci->session->userdata('stichoza');
            if(!array_key_exists($this->langTo, $stichoza)) {
                $this->ci->session->set_userdata('stichoza', array($this->langTo => array()));
            }
            // check text string in the array
            $stichoza = $this->ci->session->userdata('stichoza');
            if(array_key_exists($text, $stichoza[$this->langTo])) {
                return $stichoza[$this->langTo][$text];
            } else {
                if(!empty($proxy)) {
                    $result = $gTranslate->setHttpOption(['proxy' => 'tcp://' . $proxy])->setSource($this->langFrom)->setTarget($this->langTo)->translate($text);
                } else {
                    $result = $gTranslate->setSource($this->langFrom)->setTarget($this->langTo)->translate($text);
                }
                if(!empty($result)) { // API limit exceeded
                    $result = $this->replace_special($result, '% s', ' %s');
                    $result = $this->replace_special($result, '% S', ' %s');
                    $result = $this->replace_special($result, '% d', '%d');
                    $result = $this->replace_special($result, '</ ', '</');
                    $stichoza[$this->langTo][$text] = $result;
                    $this->ci->session->set_userdata('stichoza', $stichoza);
                }
                return $this->lastResult = $result;
            }
        } else {
            return false;
        }
    }
    
    /**
     * Text translate using: https://github.com/Stichoza/google-translate-php
     * @param string $text Text to be translated
     *
     * @param string $proxy Google has become stricter, and it seems like they keep lowering the number of allowed requests per IP per a certain amount of time.
     * Try sending less requests to stay under the radar, or change your IP frequently
     *
     * @return string|bool  Translated text OR false if failure
     */
    public function translate_stichoza($text, $proxy = null) {
        if (version_compare(PHP_VERSION, '5.5.0') >= 0) {
            $gTranslate = new GoogleTranslate();
            /*
             // use session to avoid conversion of the same text again and again
             // session does not exist, set it
             if(!$this->ci->session->userdata('stichoza')) {
             $this->ci->session->set_userdata('stichoza', array($this->langTo => array()));
             }
             // the session has a limit, so do not use it, zend cache is used for this task
             // session exists but language is not in the session
             $stichoza = (array) $this->ci->session->userdata('stichoza');
             if(!array_key_exists($this->langTo, $stichoza)) {
             $this->ci->session->set_userdata('stichoza', array($this->langTo => array()));
             }
             // check text string in the array
             $stichoza = $this->ci->session->userdata('stichoza');
             */
            // $stichoza = $this->ci->session->unset_userdata('stichoza');
            // $stichoza = (array) $this->ci->session->userdata('stichoza');
            $contents = $this->save_read_temp($this->langTo, '', true);
            if(array_key_exists($text, $contents)) {
                return $contents[$text];
            } else {
                if(!empty($proxy)) {
                    $result = $gTranslate->setHttpOption(['proxy' => 'tcp://' . $proxy])->setSource($this->langFrom)->setTarget($this->langTo)->translate($text);
                } else {
                    //try{
                    $result = $gTranslate->setSource($this->langFrom)->setTarget($this->langTo)->translate($text);
                    /* } catch(Exception $e) {
                     $result = $text;
                     }*/
                }
                if(!empty($result)) { // API limit exceeded
                    $result = $this->replace_special($result, '% s', ' %s');
                    $result = $this->replace_special($result, '% S', ' %s');
                    $result = $this->replace_special($result, '% d', '%d');
                    $result = $this->replace_special($result, '</ ', '</');
                    // $stichoza[$this->langTo][$text] = $result;
                    $this->save_read_temp($this->langTo, json_encode(array($text => $result)));
                    // $this->ci->session->set_userdata('stichoza', $stichoza);
                }
                return $this->lastResult = $result;
            }
        } else {
            return false;
        }
    }
    
    /**
     * The Session Variable has a limit to hold data
     * i.e. the file method is used
     * All translations are saved in a temp file and retrieved from there to avoid unnecessary lookup.
     * @param string $langTo - Name of the language file
     * @param string $data
     * @param boolean $readFile
     */
    private function __save_read_temp($langTo, $data, $readFile = false){
        $langPath = FCPATH . "assets/tmp/translations/" . $langTo . '.txt';
        if($readFile) {
            if(file_exists($langPath)) {
                $lines = file($langPath);
                $contents = array();
                if(!empty($lines)) {
                    foreach($lines as $line) {
                        if(!empty($line)) {
                            $line = (array) json_decode($line, true);
                            $contents = array_merge($contents, $line);
                        }
                    }
                }
                return $contents;
            } else {
                return array();
            }
        } else {
            $mode = (!file_exists($langPath)) ? 'w':'a';
            $langFile = fopen($langPath, $mode);
            fwrite($langFile, "\r\n". $data);
            fclose($langFile);
        }
    }
    
    /**
     * Translate Text using GoClixy API server
     * @param string $text
     * @return mixed
     */
    public function translate($text) {
        $translations = $this->download_language($this->langTo);
        if(isset($translations[$text])) {
            return $translations[$text];
        } else {
            return $this->translate_text($this->langFrom, $this->langTo, $text);
        }
    }
    
    /**
     * Completely translate all english language strings to a specific language
     * Upload all english text string on server one by one and perform the translation
     * @return boolean
     */
    public function bulk_translate($target = 'fr') {
        $this->ci->load->config('translate/langcodes');
        $this->ci->load->helper('translate/languages');
        $this->ci->load->library('translate/translate_lib');
        $allLangFiles = list_lang_files();
        $core_files = $allLangFiles['core'];
        $custom_files = $allLangFiles['custom'];
        $allLangFiles = array_merge($core_files, $custom_files);
        if(empty($allLangFiles)) {
            return false;
        }
        $origLang = 'english';
        $source = 'en';
        $translated_text = $this->download_language($target);
        // $this->save_read_temp($target, null, true);
        foreach ($allLangFiles as $langFile) {
            // Load the original language file
            $strings = load_lang_file($langFile, $origLang);
            if($strings) {
                foreach($strings as $key => $text) {
                    if(!array_key_exists($text, $translated_text)) {
                        $this->translate_text($source, $target, $text);
                    }
                }
            }
        }
    }
    
    /**
     * All translations are done using the GoClixy API server
     * @param string $source
     * @param string $target
     * @param string $text
     */
    private function translate_text($source, $target, $text) {
        // set post fields
        $post = [
            'source' => $source,
            'target' => $target,
            'text'   => $text,
        ];
        
        $ch = curl_init('https://api.goclixy.com/rest/translations');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
        
        // execute!
        $response = curl_exec($ch);
        
        // close the connection, release resources used
        curl_close($ch);
        
        // do anything you want with your response
        return json_decode($response ?? '');
    }
    
    /**
     * Download language files from the GoClixy API site
     * @param string $url
     */
    public function download_language($target_lang) {
        $data = array();
        if($target_lang != 'en') {
            $cacheID = 'translations_' . $target_lang;
            if(!$data = $this->gc_translate_cache->load($cacheID)) {
                $url = "https://api.goclixy.com/rest/translation?target=" . $target_lang;
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL,$url);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
                $result = curl_exec($ch);
                curl_close($ch);
                $data = json_decode($result ?? '', true);
                if(!empty($data)) {
                    $this->save_read_temp($target_lang, $result);
                }
                $this->gc_translate_cache->save($data, $cacheID, array('translations'));
            }
        }
        return $data;
    }
    
    /**
     * All translations are saved in a temp file and retrieved from there to avoid unnecessary API calls.
     * @param string $langTo - Name of the language file
     * @param string $data
     * @param boolean $readFile
     */
    private function save_read_temp($langTo, $data, $readFile = false) {
        $langPath = FCPATH . "assets/tmp/translations/" . $langTo . '.txt';
        if($readFile) {
            if(file_exists($langPath)) {
                $lines = file($langPath);
                $contents = json_decode($lines[0] ?? '');
                return $contents;
            } else {
                return array();
            }
        } else {
            if(file_exists($langPath)) {
                unlink($langPath);
            }
            $mode = (!file_exists($langPath)) ? 'w':'a';
            $langFile = fopen($langPath, $mode);
            fwrite($langFile, "\r\n". $data);
            fclose($langFile);
        }
    }
    
    /**
     * Keep special characters format
     * @param string $text
     * @param string $find
     * @param string $replace
     * @return string
     */
    private function replace_special($text, $find, $replace) {
        if (strpos($text, $find) !== false) {
            $text = str_replace($find, $replace, $text);
        }
        return $text;
    }
}